# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from os_ken.controller import handler
from os_ken.controller import event

LOG = logging.getLogger(__name__)


class EventSwitchBase(event.EventBase):
    def __init__(self, switch):
        super(EventSwitchBase, self).__init__()
        self.switch = switch

    def __str__(self):
        return '%s<dpid=%s, %s ports>' % \
            (self.__class__.__name__,
             self.switch.dp.id, len(self.switch.ports))


class EventSwitchEnter(EventSwitchBase):
    def __init__(self, switch):
        super(EventSwitchEnter, self).__init__(switch)


class EventSwitchLeave(EventSwitchBase):
    def __init__(self, switch):
        super(EventSwitchLeave, self).__init__(switch)


class EventSwitchReconnected(EventSwitchBase):
    def __init__(self, switch):
        super(EventSwitchReconnected, self).__init__(switch)


class EventPortBase(event.EventBase):
    def __init__(self, port):
        super(EventPortBase, self).__init__()
        self.port = port

    def __str__(self):
        return '%s<%s>' % (self.__class__.__name__, self.port)


class EventPortAdd(EventPortBase):
    def __init__(self, port):
        super(EventPortAdd, self).__init__(port)


class EventPortDelete(EventPortBase):
    def __init__(self, port):
        super(EventPortDelete, self).__init__(port)


class EventPortModify(EventPortBase):
    def __init__(self, port):
        super(EventPortModify, self).__init__(port)


class EventSwitchRequest(event.EventRequestBase):
    # If dpid is None, reply all list
    def __init__(self, dpid=None):
        super(EventSwitchRequest, self).__init__()
        self.dst = 'switches'
        self.dpid = dpid

    def __str__(self):
        return 'EventSwitchRequest<src=%s, dpid=%s>' % \
            (self.src, self.dpid)


class EventSwitchReply(event.EventReplyBase):
    def __init__(self, dst, switches):
        super(EventSwitchReply, self).__init__(dst)
        self.switches = switches

    def __str__(self):
        return 'EventSwitchReply<dst=%s, %s>' % \
            (self.dst, self.switches)


class EventLinkBase(event.EventBase):
    def __init__(self, link):
        super(EventLinkBase, self).__init__()
        self.link = link

    def __str__(self):
        return '%s<%s>' % (self.__class__.__name__, self.link)


class EventLinkAdd(EventLinkBase):
    def __init__(self, link):
        super(EventLinkAdd, self).__init__(link)


class EventLinkDelete(EventLinkBase):
    def __init__(self, link):
        super(EventLinkDelete, self).__init__(link)


class EventLinkRequest(event.EventRequestBase):
    # If dpid is None, reply all list
    def __init__(self, dpid=None):
        super(EventLinkRequest, self).__init__()
        self.dst = 'switches'
        self.dpid = dpid

    def __str__(self):
        return 'EventLinkRequest<src=%s, dpid=%s>' % \
            (self.src, self.dpid)


class EventLinkReply(event.EventReplyBase):
    def __init__(self, dst, dpid, links):
        super(EventLinkReply, self).__init__(dst)
        self.dpid = dpid
        self.links = links

    def __str__(self):
        return 'EventLinkReply<dst=%s, dpid=%s, links=%s>' % \
            (self.dst, self.dpid, len(self.links))


class EventHostRequest(event.EventRequestBase):
    # if dpid is None, replay all hosts
    def __init__(self, dpid=None):
        super(EventHostRequest, self).__init__()
        self.dst = 'switches'
        self.dpid = dpid

    def __str__(self):
        return 'EventHostRequest<src=%s, dpid=%s>' % \
            (self.src, self.dpid)


class EventHostReply(event.EventReplyBase):
    def __init__(self, dst, dpid, hosts):
        super(EventHostReply, self).__init__(dst)
        self.dpid = dpid
        self.hosts = hosts

    def __str__(self):
        return 'EventHostReply<dst=%s, dpid=%s, hosts=%s>' % \
            (self.dst, self.dpid, len(self.hosts))


class EventHostBase(event.EventBase):
    def __init__(self, host):
        super(EventHostBase, self).__init__()
        self.host = host

    def __str__(self):
        return '%s<%s>' % (self.__class__.__name__, self.host)


class EventHostAdd(EventHostBase):
    def __init__(self, host):
        super(EventHostAdd, self).__init__(host)


# Note: Currently, EventHostDelete will never be raised, because we have no
# appropriate way to detect the disconnection of hosts. Just defined for
# future use.
class EventHostDelete(EventHostBase):
    def __init__(self, host):
        super(EventHostDelete, self).__init__(host)


class EventHostMove(event.EventBase):
    def __init__(self, src, dst):
        super(EventHostMove, self).__init__()
        self.src = src
        self.dst = dst

    def __str__(self):
        return '%s<src=%s, dst=%s>' % (
            self.__class__.__name__, self.src, self.dst)


handler.register_service('os_ken.topology.switches')



# Copyright (C) 2011 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2011 Isaku Yamahata <yamahata at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from os_ken.lib import addrconv

if six.PY3:
    _ord = int
else:
    _ord = ord

# string representation
HADDR_PATTERN = r'([0-9a-f]{2}:){5}[0-9a-f]{2}'

DONTCARE = b'\x00' * 6
BROADCAST = b'\xff' * 6
DONTCARE_STR = '00:00:00:00:00:00'
BROADCAST_STR = 'ff:ff:ff:ff:ff:ff'
MULTICAST = 'fe:ff:ff:ff:ff:ff'
UNICAST = '01:00:00:00:00:00'


def is_multicast(addr):
    return bool(_ord(addr[0]) & 0x01)


def haddr_to_str(addr):
    """Format mac address in internal representation into human readable
    form"""
    if addr is None:
        return 'None'
    try:
        return addrconv.mac.bin_to_text(addr)
    except:
        raise AssertionError


def haddr_to_int(addr):
    """Convert mac address string in human readable format into
    integer value"""
    try:
        return int(addr.replace(':', ''), 16)
    except:
        raise ValueError


def haddr_to_bin(string):
    """Parse mac address string in human readable format into
    internal representation"""
    try:
        return addrconv.mac.text_to_bin(string)
    except:
        raise ValueError


def haddr_bitand(addr, mask):
    return b''.join(six.int2byte(_ord(a) & _ord(m)) for (a, m)
                    in zip(addr, mask))



# Copyright (C) 2013,2014 Nippon Telegraph and Telephone Corporation.
# Copyright (C) 2013,2014 YAMAMOTO Takashi <yamamoto at valinux co jp>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import sys

import unittest
from nose.tools import eq_
from nose.tools import ok_

from os_ken.utils import binary_str
from os_ken.lib import pcaplib
from os_ken.lib.packet import packet
from os_ken.lib.packet import bgp
from os_ken.lib.packet import afi
from os_ken.lib.packet import safi


LOG = logging.getLogger(__name__)

BGP4_PACKET_DATA_DIR = os.path.join(
    os.path.dirname(sys.modules[__name__].__file__), '../../packet_data/bgp4/')

PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT = (
    bgp.BGPPathAttributePmsiTunnel.TYPE_NO_TUNNEL_INFORMATION_PRESENT
)
PMSI_TYPE_INGRESS_REPLICATION = (
    bgp.BGPPathAttributePmsiTunnel.TYPE_INGRESS_REPLICATION
)

RULES_BASE = [
    # port='>=8000'
    bgp.FlowSpecPort(
        operator=(bgp.FlowSpecPort.GT | bgp.FlowSpecPort.EQ),
        value=8000),
    # port='&<=9000'
    bgp.FlowSpecPort(
        operator=(bgp.FlowSpecPort.AND | bgp.FlowSpecPort.LT |
                  bgp.FlowSpecPort.EQ),
        value=9000),
    # port='==80'
    bgp.FlowSpecPort(operator=bgp.FlowSpecPort.EQ, value=80),
    # dst_port=8080
    bgp.FlowSpecDestPort(operator=bgp.FlowSpecDestPort.EQ, value=8080),
    # dst_port='>9000'
    bgp.FlowSpecDestPort(operator=bgp.FlowSpecDestPort.GT, value=9000),
    # dst_port='&<9050'
    bgp.FlowSpecDestPort(
        operator=(bgp.FlowSpecDestPort.AND | bgp.FlowSpecDestPort.LT),
        value=9050),
    # dst_port='<=1000'
    bgp.FlowSpecDestPort(
        operator=(bgp.FlowSpecDestPort.LT | bgp.FlowSpecDestPort.EQ),
        value=1000),
    # src_port='<=9090'
    bgp.FlowSpecSrcPort(
        operator=(bgp.FlowSpecSrcPort.LT | bgp.FlowSpecSrcPort.EQ),
        value=9090),
    # src_port='& >=9080'
    bgp.FlowSpecSrcPort(
        operator=(bgp.FlowSpecSrcPort.AND | bgp.FlowSpecSrcPort.GT |
                  bgp.FlowSpecSrcPort.EQ),
        value=9080),
    # src_port='<10100'
    bgp.FlowSpecSrcPort(
        operator=bgp.FlowSpecSrcPort.LT, value=10100),
    # src_port='>10000'
    bgp.FlowSpecSrcPort(
        operator=(bgp.FlowSpecSrcPort.AND | bgp.FlowSpecSrcPort.GT),
        value=10000),
    # icmp_type=0
    bgp.FlowSpecIcmpType(operator=bgp.FlowSpecIcmpType.EQ, value=0),
    # icmp_code=6
    bgp.FlowSpecIcmpCode(operator=bgp.FlowSpecIcmpCode.EQ, value=6),
    # tcp_flags='ACK+FIN'
    bgp.FlowSpecTCPFlags(
        operator=0,  # Partial match
        value=(bgp.FlowSpecTCPFlags.SYN | bgp.FlowSpecTCPFlags.ACK)),
    # tcp_flags='&!=URGENT'
    bgp.FlowSpecTCPFlags(
        operator=(bgp.FlowSpecTCPFlags.AND | bgp.FlowSpecTCPFlags.NOT),
        value=bgp.FlowSpecTCPFlags.URGENT),
    # packet_len=1000
    bgp.FlowSpecPacketLen(
        operator=bgp.FlowSpecPacketLen.EQ, value=1000),
    # packet_len=1100
    bgp.FlowSpecPacketLen(
        operator=(bgp.FlowSpecTCPFlags.AND | bgp.FlowSpecPacketLen.EQ),
        value=1100),
    # dscp=22
    bgp.FlowSpecDSCP(operator=bgp.FlowSpecDSCP.EQ, value=22),
    # dscp=24
    bgp.FlowSpecDSCP(operator=bgp.FlowSpecDSCP.EQ, value=24),
]

RULES_L2VPN_BASE = [
    # ether_type=0x0800
    bgp.FlowSpecEtherType(operator=bgp.FlowSpecEtherType.EQ, value=0x0800),
    # source_mac='12:34:56:78:90:AB'
    bgp.FlowSpecSourceMac(addr='12:34:56:78:90:AB', length=6),
    # dest_mac='DE:EF:C0:FF:EE:DD'
    bgp.FlowSpecDestinationMac(addr='BE:EF:C0:FF:EE:DD', length=6),
    # llc_dsap=0x42
    bgp.FlowSpecLLCDSAP(operator=bgp.FlowSpecLLCDSAP.EQ, value=0x42),
    # llc_ssap=0x42
    bgp.FlowSpecLLCSSAP(operator=bgp.FlowSpecLLCSSAP.EQ, value=0x42),
    # llc_control=100
    bgp.FlowSpecLLCControl(operator=bgp.FlowSpecLLCControl.EQ, value=100),
    # snap=0x12345
    bgp.FlowSpecSNAP(operator=bgp.FlowSpecSNAP.EQ, value=0x12345),
    # vlan_id='>4000'
    bgp.FlowSpecVLANID(operator=bgp.FlowSpecVLANID.GT, value=4000),
    # vlan_cos='>=3'
    bgp.FlowSpecVLANCoS(
        operator=(bgp.FlowSpecVLANCoS.GT | bgp.FlowSpecVLANCoS.EQ), value=3),
    # inner_vlan_id='<3000'
    bgp.FlowSpecInnerVLANID(operator=bgp.FlowSpecInnerVLANID.LT, value=3000),
    # inner_vlan_cos='<=5'
    bgp.FlowSpecInnerVLANCoS(
        operator=(bgp.FlowSpecInnerVLANCoS.LT | bgp.FlowSpecInnerVLANCoS.EQ),
        value=5),
]


class Test_bgp(unittest.TestCase):
    """ Test case for os_ken.lib.packet.bgp
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_open1(self):
        msg = bgp.BGPOpen(my_as=30000, bgp_identifier='192.0.2.1')
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        eq_(len(msg), 29)
        eq_(rest, b'')

    def test_open2(self):
        opt_param = [bgp.BGPOptParamCapabilityUnknown(cap_code=200,
                                                      cap_value=b'hoge'),
                     bgp.BGPOptParamCapabilityGracefulRestart(flags=0,
                                                              time=120,
                                                              tuples=[]),
                     bgp.BGPOptParamCapabilityRouteRefresh(),
                     bgp.BGPOptParamCapabilityCiscoRouteRefresh(),
                     bgp.BGPOptParamCapabilityMultiprotocol(
                         afi=afi.IP, safi=safi.MPLS_VPN),
                     bgp.BGPOptParamCapabilityCarryingLabelInfo(),
                     bgp.BGPOptParamCapabilityFourOctetAsNumber(
                         as_number=1234567),
                     bgp.BGPOptParamUnknown(type_=99, value=b'fuga')]
        msg = bgp.BGPOpen(my_as=30000, bgp_identifier='192.0.2.2',
                          opt_param=opt_param)
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        ok_(len(msg) > 29)
        eq_(rest, b'')

    def test_update1(self):
        msg = bgp.BGPUpdate()
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        eq_(len(msg), 23)
        eq_(rest, b'')

    def test_update2(self):
        withdrawn_routes = [bgp.BGPWithdrawnRoute(length=0,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=1,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=3,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=7,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=32,
                                                  addr='192.0.2.13')]
        mp_nlri = [
            bgp.LabelledVPNIPAddrPrefix(24, '192.0.9.0',
                                        route_dist='100:100',
                                        labels=[1, 2, 3]),
            bgp.LabelledVPNIPAddrPrefix(26, '192.0.10.192',
                                        route_dist='10.0.0.1:10000',
                                        labels=[5, 6, 7, 8]),
        ]
        mp_nlri2 = [
            bgp.LabelledIPAddrPrefix(24, '192.168.0.0', labels=[1, 2, 3])
        ]
        mp_nlri_v6 = [
            bgp.LabelledVPNIP6AddrPrefix(64, '2001:db8:1111::',
                                         route_dist='200:200',
                                         labels=[1, 2, 3]),
            bgp.LabelledVPNIP6AddrPrefix(64, '2001:db8:2222::',
                                         route_dist='10.0.0.1:10000',
                                         labels=[5, 6, 7, 8]),
        ]
        mp_nlri2_v6 = [
            bgp.LabelledIP6AddrPrefix(64, '2001:db8:3333::', labels=[1, 2, 3])
        ]
        communities = [
            bgp.BGP_COMMUNITY_NO_EXPORT,
            bgp.BGP_COMMUNITY_NO_ADVERTISE,
        ]
        ecommunities = [
            bgp.BGPTwoOctetAsSpecificExtendedCommunity(
                subtype=1, as_number=65500, local_administrator=3908876543),
            bgp.BGPFourOctetAsSpecificExtendedCommunity(
                subtype=2, as_number=10000000, local_administrator=59876),
            bgp.BGPIPv4AddressSpecificExtendedCommunity(
                subtype=3, ipv4_address='192.0.2.1',
                local_administrator=65432),
            bgp.BGPOpaqueExtendedCommunity(subtype=13, opaque=b'abcdef'),
            bgp.BGPEncapsulationExtendedCommunity(
                subtype=0x0c, tunnel_type=10),
            bgp.BGPEvpnMacMobilityExtendedCommunity(
                subtype=0, flags=0xff, sequence_number=0x11223344),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, label=b'\xFF\xFF\xFF'),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, mpls_label=0xfffff),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, vni=0xffffff),
            bgp.BGPEvpnEsImportRTExtendedCommunity(
                subtype=2, es_import="aa:bb:cc:dd:ee:ff"),
            bgp.BGPUnknownExtendedCommunity(type_=99, value=b'abcdefg'),
        ]
        path_attributes = [
            bgp.BGPPathAttributeOrigin(value=1),
            bgp.BGPPathAttributeAsPath(value=[[1000], {1001, 1002},
                                              [1003, 1004]]),
            bgp.BGPPathAttributeNextHop(value='192.0.2.199'),
            bgp.BGPPathAttributeMultiExitDisc(value=2000000000),
            bgp.BGPPathAttributeLocalPref(value=1000000000),
            bgp.BGPPathAttributeAtomicAggregate(),
            bgp.BGPPathAttributeAggregator(as_number=40000,
                                           addr='192.0.2.99'),
            bgp.BGPPathAttributeCommunities(communities=communities),
            bgp.BGPPathAttributeOriginatorId(value='10.1.1.1'),
            bgp.BGPPathAttributeClusterList(value=['1.1.1.1', '2.2.2.2']),
            bgp.BGPPathAttributeExtendedCommunities(communities=ecommunities),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
                label=b'\xFF\xFF\xFF'),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
                tunnel_id=None),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
                mpls_label=0xfffff,
                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
                    tunnel_endpoint_ip="1.1.1.1")),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
                vni=0xffffff,
                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
                    tunnel_endpoint_ip="aa:bb:cc::dd:ee:ff")),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=2,
                label=b'\xFF\xFF\xFF',
                tunnel_id=bgp.PmsiTunnelIdUnknown(value=b'test')),
            bgp.BGPPathAttributeAs4Path(value=[[1000000], {1000001, 1002},
                                               [1003, 1000004]]),
            bgp.BGPPathAttributeAs4Aggregator(as_number=100040000,
                                              addr='192.0.2.99'),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                            next_hop='1.1.1.1',
                                            nlri=mp_nlri),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP, safi=safi.MPLS_LABEL,
                                            next_hop='1.1.1.1',
                                            nlri=mp_nlri2),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP6, safi=safi.MPLS_VPN,
                                            next_hop=['2001:db8::1'],
                                            nlri=mp_nlri_v6),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP6, safi=safi.MPLS_LABEL,
                                            next_hop=['2001:db8::1',
                                                      'fe80::1'],
                                            nlri=mp_nlri2_v6),
            bgp.BGPPathAttributeMpUnreachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                              withdrawn_routes=mp_nlri),
            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300 * b'bar')
        ]
        nlri = [
            bgp.BGPNLRI(length=24, addr='203.0.113.1'),
            bgp.BGPNLRI(length=16, addr='203.0.113.0')
        ]
        msg = bgp.BGPUpdate(withdrawn_routes=withdrawn_routes,
                            path_attributes=path_attributes,
                            nlri=nlri)
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        ok_(len(msg) > 23)
        eq_(rest, b'')

    def test_keepalive(self):
        msg = bgp.BGPKeepAlive()
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        eq_(len(msg), 19)
        eq_(rest, b'')

    def test_notification(self):
        data = b'hoge'
        msg = bgp.BGPNotification(error_code=1, error_subcode=2, data=data)
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        eq_(len(msg), 21 + len(data))
        eq_(rest, b'')

    def test_route_refresh(self):
        msg = bgp.BGPRouteRefresh(afi=afi.IP, safi=safi.MPLS_VPN)
        binmsg = msg.serialize()
        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
        eq_(str(msg), str(msg2))
        eq_(len(msg), 23)
        eq_(rest, b'')

    def test_stream_parser(self):
        msgs = [
            bgp.BGPNotification(error_code=1, error_subcode=2, data=b'foo'),
            bgp.BGPNotification(error_code=3, error_subcode=4, data=b'bar'),
            bgp.BGPNotification(error_code=5, error_subcode=6, data=b'baz'),
        ]
        binmsgs = b''.join([bytes(msg.serialize()) for msg in msgs])
        sp = bgp.StreamParser()
        results = []
        for b in binmsgs:
            for m in sp.parse(b):
                results.append(m)
        eq_(str(results), str(msgs))

    def test_parser(self):
        files = [
            'bgp4-open',
            'bgp4-update',
            'bgp4-update_ipv6',
            'bgp4-update_vpnv6',
            'bgp4-keepalive',
            'evpn_esi_arbitrary',
            'evpn_esi_lacp',
            'evpn_esi_l2_bridge',
            'evpn_esi_mac_base',
            'evpn_esi_router_id',
            'evpn_esi_as_based',
            'evpn_nlri_eth_a-d',
            'evpn_nlri_mac_ip_ad',
            'evpn_nlri_inc_multi_eth_tag',
            'evpn_nlri_eth_seg',
            'evpn_nlri_ip_prefix',
            'flowspec_nlri_ipv4',
            'flowspec_nlri_vpn4',
            'flowspec_nlri_ipv6',
            'flowspec_nlri_vpn6',
            'flowspec_nlri_l2vpn',
            'flowspec_action_traffic_rate',
            'flowspec_action_traffic_action',
            'flowspec_action_redirect',
            'flowspec_action_traffic_marking',
        ]

        for f in files:
            LOG.debug('*** testing %s ...', f)
            for _, buf in pcaplib.Reader(
                    open(BGP4_PACKET_DATA_DIR + f + '.pcap', 'rb')):
                # Checks if BGP message can be parsed as expected.
                pkt = packet.Packet(buf)
                ok_(isinstance(pkt.protocols[-1], bgp.BGPMessage),
                    'Failed to parse BGP message: %s' % pkt)

                # Checks if BGP message can be serialized as expected.
                pkt.serialize()
                eq_(buf, pkt.data,
                    "b'%s' != b'%s'" % (binary_str(buf), binary_str(pkt.data)))

    def test_vlan_action_parser(self):
        action = bgp.BGPFlowSpecVlanActionCommunity(
            actions_1=(bgp.BGPFlowSpecVlanActionCommunity.POP |
                       bgp.BGPFlowSpecVlanActionCommunity.SWAP),
            vlan_1=3000,
            cos_1=3,
            actions_2=bgp.BGPFlowSpecVlanActionCommunity.PUSH,
            vlan_2=4000,
            cos_2=2,
        )
        binmsg = action.serialize()
        msg, rest = bgp.BGPFlowSpecVlanActionCommunity.parse(binmsg)
        eq_(str(action), str(msg))
        eq_(rest, b'')

    def test_tpid_action_parser(self):
        action = bgp.BGPFlowSpecTPIDActionCommunity(
            actions=(bgp.BGPFlowSpecTPIDActionCommunity.TI |
                     bgp.BGPFlowSpecTPIDActionCommunity.TO),
            tpid_1=5,
            tpid_2=6,
        )
        binmsg = action.serialize()
        msg, rest = bgp.BGPFlowSpecTPIDActionCommunity.parse(binmsg)
        eq_(str(action), str(msg))
        eq_(rest, b'')

    def test_json1(self):
        opt_param = [bgp.BGPOptParamCapabilityUnknown(cap_code=200,
                                                      cap_value=b'hoge'),
                     bgp.BGPOptParamCapabilityRouteRefresh(),
                     bgp.BGPOptParamCapabilityMultiprotocol(
                         afi=afi.IP, safi=safi.MPLS_VPN),
                     bgp.BGPOptParamCapabilityFourOctetAsNumber(
                         as_number=1234567),
                     bgp.BGPOptParamUnknown(type_=99, value=b'fuga')]
        msg1 = bgp.BGPOpen(my_as=30000, bgp_identifier='192.0.2.2',
                           opt_param=opt_param)
        jsondict = msg1.to_jsondict()
        msg2 = bgp.BGPOpen.from_jsondict(jsondict['BGPOpen'])
        eq_(str(msg1), str(msg2))

    def test_json2(self):
        withdrawn_routes = [bgp.BGPWithdrawnRoute(length=0,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=1,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=3,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=7,
                                                  addr='192.0.2.13'),
                            bgp.BGPWithdrawnRoute(length=32,
                                                  addr='192.0.2.13')]
        mp_nlri = [
            bgp.LabelledVPNIPAddrPrefix(24, '192.0.9.0',
                                        route_dist='100:100',
                                        labels=[1, 2, 3]),
            bgp.LabelledVPNIPAddrPrefix(26, '192.0.10.192',
                                        route_dist='10.0.0.1:10000',
                                        labels=[5, 6, 7, 8]),
        ]
        mp_nlri2 = [
            bgp.LabelledIPAddrPrefix(24, '192.168.0.0', labels=[1, 2, 3])
        ]
        mp_nlri_v6 = [
            bgp.LabelledVPNIP6AddrPrefix(64, '2001:db8:1111::',
                                         route_dist='200:200',
                                         labels=[1, 2, 3]),
            bgp.LabelledVPNIP6AddrPrefix(64, '2001:db8:2222::',
                                         route_dist='10.0.0.1:10000',
                                         labels=[5, 6, 7, 8]),
        ]
        mp_nlri2_v6 = [
            bgp.LabelledIP6AddrPrefix(64, '2001:db8:3333::', labels=[1, 2, 3])
        ]
        communities = [
            bgp.BGP_COMMUNITY_NO_EXPORT,
            bgp.BGP_COMMUNITY_NO_ADVERTISE,
        ]
        ecommunities = [
            bgp.BGPTwoOctetAsSpecificExtendedCommunity(
                subtype=1, as_number=65500, local_administrator=3908876543),
            bgp.BGPFourOctetAsSpecificExtendedCommunity(
                subtype=2, as_number=10000000, local_administrator=59876),
            bgp.BGPIPv4AddressSpecificExtendedCommunity(
                subtype=3, ipv4_address='192.0.2.1',
                local_administrator=65432),
            bgp.BGPOpaqueExtendedCommunity(subtype=13, opaque=b'abcdef'),
            bgp.BGPEncapsulationExtendedCommunity(
                subtype=0x0c, tunnel_type=10),
            bgp.BGPEvpnMacMobilityExtendedCommunity(
                subtype=0, flags=0xff, sequence_number=0x11223344),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, label=b'\xFF\xFF\xFF'),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, mpls_label=0xfffff),
            bgp.BGPEvpnEsiLabelExtendedCommunity(
                subtype=1, flags=0xff, vni=0xffffff),
            bgp.BGPEvpnEsImportRTExtendedCommunity(
                subtype=2, es_import="aa:bb:cc:dd:ee:ff"),
            bgp.BGPUnknownExtendedCommunity(type_=99, value=b'abcdefg'),
        ]
        path_attributes = [
            bgp.BGPPathAttributeOrigin(value=1),
            bgp.BGPPathAttributeAsPath(value=[[1000], {1001, 1002},
                                              [1003, 1004]]),
            bgp.BGPPathAttributeNextHop(value='192.0.2.199'),
            bgp.BGPPathAttributeMultiExitDisc(value=2000000000),
            bgp.BGPPathAttributeLocalPref(value=1000000000),
            bgp.BGPPathAttributeAtomicAggregate(),
            bgp.BGPPathAttributeAggregator(as_number=40000,
                                           addr='192.0.2.99'),
            bgp.BGPPathAttributeCommunities(communities=communities),
            bgp.BGPPathAttributeExtendedCommunities(communities=ecommunities),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
                label=b'\xFF\xFF\xFF'),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
                tunnel_id=None),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
                mpls_label=0xfffff,
                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
                    tunnel_endpoint_ip="1.1.1.1")),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
                vni=0xffffff,
                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
                    tunnel_endpoint_ip="aa:bb:cc::dd:ee:ff")),
            bgp.BGPPathAttributePmsiTunnel(
                pmsi_flags=1,
                tunnel_type=2,
                label=b'\xFF\xFF\xFF',
                tunnel_id=bgp.PmsiTunnelIdUnknown(value=b'test')),
            bgp.BGPPathAttributeAs4Path(value=[[1000000], {1000001, 1002},
                                               [1003, 1000004]]),
            bgp.BGPPathAttributeAs4Aggregator(as_number=100040000,
                                              addr='192.0.2.99'),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                            next_hop='1.1.1.1',
                                            nlri=mp_nlri),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP, safi=safi.MPLS_LABEL,
                                            next_hop='1.1.1.1',
                                            nlri=mp_nlri2),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP6, safi=safi.MPLS_VPN,
                                            next_hop=['2001:db8::1'],
                                            nlri=mp_nlri_v6),
            bgp.BGPPathAttributeMpReachNLRI(afi=afi.IP6, safi=safi.MPLS_LABEL,
                                            next_hop=['2001:db8::1',
                                                      'fe80::1'],
                                            nlri=mp_nlri2_v6),
            bgp.BGPPathAttributeMpUnreachNLRI(afi=afi.IP, safi=safi.MPLS_VPN,
                                              withdrawn_routes=mp_nlri),
            bgp.BGPPathAttributeUnknown(flags=0, type_=100, value=300 * b'bar')
        ]
        nlri = [
            bgp.BGPNLRI(length=24, addr='203.0.113.1'),
            bgp.BGPNLRI(length=16, addr='203.0.113.0')
        ]
        msg1 = bgp.BGPUpdate(withdrawn_routes=withdrawn_routes,
                             path_attributes=path_attributes,
                             nlri=nlri)
        jsondict = msg1.to_jsondict()
        msg2 = bgp.BGPUpdate.from_jsondict(jsondict['BGPUpdate'])
        eq_(str(msg1), str(msg2))

    def test_flowspec_user_interface_ipv4(self):
        rules = RULES_BASE + [
            # dst_prefix='10.0.0.0/24
            bgp.FlowSpecDestPrefix(addr='10.0.0.0', length=24),
            # src_prefix='20.0.0.1/24'
            bgp.FlowSpecSrcPrefix(addr='20.0.0.0', length=24),
            # ip_proto='6'
            bgp.FlowSpecIPProtocol(
                operator=bgp.FlowSpecIPProtocol.EQ, value=6),
            # fragment='LF'
            bgp.FlowSpecFragment(
                operator=0,  # Partial match
                value=bgp.FlowSpecFragment.LF),
            # fragment='==FF'
            bgp.FlowSpecFragment(
                operator=bgp.FlowSpecFragment.MATCH,
                value=bgp.FlowSpecFragment.FF),
            # fragment='&==ISF'
            bgp.FlowSpecFragment(
                operator=(bgp.FlowSpecFragment.AND |
                          bgp.FlowSpecFragment.MATCH),
                value=bgp.FlowSpecFragment.ISF),
            # fragment='!=DF'
            bgp.FlowSpecFragment(
                operator=bgp.FlowSpecFragment.NOT,
                value=bgp.FlowSpecFragment.DF)
        ]

        msg = bgp.FlowSpecIPv4NLRI.from_user(
            dst_prefix='10.0.0.0/24',
            src_prefix='20.0.0.0/24',
            ip_proto='6',
            port='>=8000 & <=9000 | ==80',
            dst_port='8080 >9000&<9050 | <=1000',
            src_port='<=9090 & >=9080 <10100 & >10000',
            icmp_type=0,
            icmp_code=6,
            tcp_flags='SYN+ACK & !=URGENT',
            packet_len='1000 & 1100',
            dscp='22 24',
            fragment='LF ==FF&==ISF | !=DF')
        msg2 = bgp.FlowSpecIPv4NLRI(rules=rules)
        binmsg = msg.serialize()
        binmsg2 = msg2.serialize()
        eq_(str(msg), str(msg2))
        eq_(binary_str(binmsg), binary_str(binmsg2))
        msg3, rest = bgp.FlowSpecIPv4NLRI.parser(binmsg)
        eq_(str(msg), str(msg3))
        eq_(rest, b'')

    def test_flowspec_user_interface_vpv4(self):
        rules = RULES_BASE + [
            # dst_prefix='10.0.0.0/24
            bgp.FlowSpecDestPrefix(addr='10.0.0.0', length=24),
            # src_prefix='20.0.0.1/24'
            bgp.FlowSpecSrcPrefix(addr='20.0.0.0', length=24),
            # ip_proto='6'
            bgp.FlowSpecIPProtocol(
                operator=bgp.FlowSpecIPProtocol.EQ, value=6),
            # fragment='LF'
            bgp.FlowSpecFragment(
                operator=0,  # Partial match
                value=bgp.FlowSpecFragment.LF),
            # fragment='==FF'
            bgp.FlowSpecFragment(
                operator=bgp.FlowSpecFragment.MATCH,
                value=bgp.FlowSpecFragment.FF),
            # fragment='&==ISF'
            bgp.FlowSpecFragment(
                operator=(bgp.FlowSpecFragment.AND |
                          bgp.FlowSpecFragment.MATCH),
                value=bgp.FlowSpecFragment.ISF),
            # fragment='!=DF'
            bgp.FlowSpecFragment(
                operator=bgp.FlowSpecFragment.NOT,
                value=bgp.FlowSpecFragment.DF)
        ]
        msg = bgp.FlowSpecVPNv4NLRI.from_user(
            route_dist='65001:250',
            dst_prefix='10.0.0.0/24',
            src_prefix='20.0.0.0/24',
            ip_proto='6',
            port='>=8000 & <=9000 | ==80',
            dst_port='8080 >9000&<9050 | <=1000',
            src_port='<=9090 & >=9080 <10100 & >10000',
            icmp_type=0,
            icmp_code=6,
            tcp_flags='SYN+ACK & !=URGENT',
            packet_len='1000 & 1100',
            dscp='22 24',
            fragment='LF ==FF&==ISF | !=DF')
        msg2 = bgp.FlowSpecVPNv4NLRI(route_dist='65001:250', rules=rules)
        binmsg = msg.serialize()
        binmsg2 = msg2.serialize()
        eq_(str(msg), str(msg2))
        eq_(binary_str(binmsg), binary_str(binmsg2))
        msg3, rest = bgp.FlowSpecVPNv4NLRI.parser(binmsg)
        eq_(str(msg), str(msg3))
        eq_(rest, b'')

    def test_flowspec_user_interface_ipv6(self):
        rules = RULES_BASE + [
            # dst_prefix='2001:2/128/32'
            bgp.FlowSpecIPv6DestPrefix(
                addr='2001::2', offset=32, length=128),
            # src_prefix='3002::3/128'
            bgp.FlowSpecIPv6SrcPrefix(
                addr='3002::3', length=128),
            # ip_proto='6'
            bgp.FlowSpecNextHeader(
                operator=bgp.FlowSpecNextHeader.EQ, value=6),
            # fragment='LF'
            bgp.FlowSpecIPv6Fragment(
                operator=0,  # Partial match
                value=bgp.FlowSpecFragment.LF),
            # fragment='==FF'
            bgp.FlowSpecIPv6Fragment(
                operator=bgp.FlowSpecFragment.MATCH,
                value=bgp.FlowSpecFragment.FF),
            # fragment='&==ISF'
            bgp.FlowSpecIPv6Fragment(
                operator=(bgp.FlowSpecFragment.AND |
                          bgp.FlowSpecFragment.MATCH),
                value=bgp.FlowSpecFragment.ISF),
            # fragment='!=LF'
            bgp.FlowSpecIPv6Fragment(
                operator=bgp.FlowSpecFragment.NOT,
                value=bgp.FlowSpecFragment.LF),
            # flowlabel='100'
            bgp.FlowSpecIPv6FlowLabel(
                operator=bgp.FlowSpecIPv6FlowLabel.EQ,
                value=100),
        ]
        msg = bgp.FlowSpecIPv6NLRI.from_user(
            dst_prefix='2001::2/128/32',
            src_prefix='3002::3/128',
            next_header='6',
            port='>=8000 & <=9000 | ==80',
            dst_port='8080 >9000&<9050 | <=1000',
            src_port='<=9090 & >=9080 <10100 & >10000',
            icmp_type=0,
            icmp_code=6,
            tcp_flags='SYN+ACK & !=URGENT',
            packet_len='1000 & 1100',
            dscp='22 24',
            fragment='LF ==FF&==ISF | !=LF',
            flow_label=100,
        )
        msg2 = bgp.FlowSpecIPv6NLRI(rules=rules)
        binmsg = msg.serialize()
        binmsg2 = msg2.serialize()
        eq_(str(msg), str(msg2))
        eq_(binary_str(binmsg), binary_str(binmsg2))
        msg3, rest = bgp.FlowSpecIPv6NLRI.parser(binmsg)
        eq_(str(msg), str(msg3))
        eq_(rest, b'')

    def test_flowspec_user_interface_vpnv6(self):
        rules = RULES_BASE + [
            # dst_prefix='2001:2/128/32'
            bgp.FlowSpecIPv6DestPrefix(
                addr='2001::2', offset=32, length=128),
            # src_prefix='3002::3/128'
            bgp.FlowSpecIPv6SrcPrefix(
                addr='3002::3', length=128),
            # ip_proto='6'
            bgp.FlowSpecNextHeader(
                operator=bgp.FlowSpecNextHeader.EQ, value=6),
            # fragment='LF'
            bgp.FlowSpecIPv6Fragment(
                operator=0,  # Partial match
                value=bgp.FlowSpecFragment.LF),
            # fragment='==FF'
            bgp.FlowSpecIPv6Fragment(
                operator=bgp.FlowSpecFragment.MATCH,
                value=bgp.FlowSpecFragment.FF),
            # fragment='&==ISF'
            bgp.FlowSpecIPv6Fragment(
                operator=(bgp.FlowSpecFragment.AND |
                          bgp.FlowSpecFragment.MATCH),
                value=bgp.FlowSpecFragment.ISF),
            # fragment='!=LF'
            bgp.FlowSpecIPv6Fragment(
                operator=bgp.FlowSpecFragment.NOT,
                value=bgp.FlowSpecFragment.LF),
            # flowlabel='100'
            bgp.FlowSpecIPv6FlowLabel(
                operator=bgp.FlowSpecIPv6FlowLabel.EQ,
                value=100),
        ]
        msg = bgp.FlowSpecVPNv6NLRI.from_user(
            route_dist='65001:250',
            dst_prefix='2001::2/128/32',
            src_prefix='3002::3/128',
            next_header='6',
            port='>=8000 & <=9000 | ==80',
            dst_port='8080 >9000&<9050 | <=1000',
            src_port='<=9090 & >=9080 <10100 & >10000',
            icmp_type=0,
            icmp_code=6,
            tcp_flags='SYN+ACK & !=URGENT',
            packet_len='1000 & 1100',
            dscp='22 24',
            fragment='LF ==FF&==ISF | !=LF',
            flow_label=100,
        )
        msg2 = bgp.FlowSpecVPNv6NLRI(route_dist='65001:250', rules=rules)
        binmsg = msg.serialize()
        binmsg2 = msg2.serialize()
        eq_(str(msg), str(msg2))
        eq_(binary_str(binmsg), binary_str(binmsg2))
        msg3, rest = bgp.FlowSpecVPNv6NLRI.parser(binmsg)
        eq_(str(msg), str(msg3))
        eq_(rest, b'')

    def test_flowspec_user_interface_l2vpn(self):
        rules = RULES_L2VPN_BASE
        msg = bgp.FlowSpecL2VPNNLRI.from_user(
            route_dist='65001:250',
            ether_type=0x0800,
            src_mac='12:34:56:78:90:AB',
            dst_mac='BE:EF:C0:FF:EE:DD',
            llc_dsap=0x42,
            llc_ssap=0x42,
            llc_control=100,
            snap=0x12345,
            vlan_id='>4000',
            vlan_cos='>=3',
            inner_vlan_id='<3000',
            inner_vlan_cos='<=5',
        )
        msg2 = bgp.FlowSpecL2VPNNLRI(route_dist='65001:250', rules=rules)
        binmsg = msg.serialize()
        binmsg2 = msg2.serialize()
        eq_(str(msg), str(msg2))
        eq_(binary_str(binmsg), binary_str(binmsg2))
        msg3, rest = bgp.FlowSpecL2VPNNLRI.parser(binmsg)
        eq_(str(msg), str(msg3))
        eq_(rest, b'')



# Copyright (C) 2014 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
 Api for operator. Mainly commands to build CLI and
 operator interface around them.
"""
import logging

from os_ken.services.protocols.bgp.api.base import ApiException
from os_ken.services.protocols.bgp.api.base import register
from os_ken.services.protocols.bgp.api.rpc_log_handler import RpcLogHandler
from os_ken.services.protocols.bgp.operator.command import Command
from os_ken.services.protocols.bgp.operator.command import STATUS_ERROR
from os_ken.services.protocols.bgp.operator.commands.clear import ClearCmd
from os_ken.services.protocols.bgp.operator.commands.set import SetCmd
from os_ken.services.protocols.bgp.operator.commands.show import ShowCmd
from os_ken.services.protocols.bgp.operator.internal_api import InternalApi

LOG = logging.getLogger('bgpspeaker.api.rtconf')

DEFAULT_LOG_FORMAT = '%(asctime)s %(levelname)s %(message)s'


def _init_log_handler():
    log_handler = RpcLogHandler()
    log_handler.setLevel(logging.ERROR)
    log_handler.formatter = logging.Formatter(DEFAULT_LOG_FORMAT)
    return log_handler


INTERNAL_API = InternalApi(_init_log_handler())


class RootCmd(Command):
    subcommands = {
        'show': ShowCmd,
        'set': SetCmd,
        'clear': ClearCmd}


def operator_run(cmd, **kwargs):
    params = kwargs.get('params', [])
    fmt = kwargs.get('format', 'json')
    root = RootCmd(api=INTERNAL_API, resp_formatter_name=fmt)
    ret, _ = root([cmd] + params)
    if ret.status == STATUS_ERROR:
        raise ApiException(str(ret.value))
    return ret.value


@register(name="operator.show")
def operator_show(**kwargs):
    return operator_run('show', **kwargs)


@register(name="operator.set")
def operator_set(**kwargs):
    return operator_run('set', **kwargs)


@register(name="operator.clear")
def operator_clear(**kwargs):
    return operator_run('clear', **kwargs)



import logging
import netaddr

from os_ken.services.protocols.bgp.base import SUPPORTED_GLOBAL_RF
from os_ken.services.protocols.bgp.model import OutgoingRoute
from os_ken.services.protocols.bgp.peer import Peer
from os_ken.lib.packet.bgp import BGPPathAttributeCommunities
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_MULTI_EXIT_DISC
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_COMMUNITIES
from os_ken.lib.packet.bgp import RF_RTC_UC
from os_ken.lib.packet.bgp import RouteTargetMembershipNLRI
from os_ken.services.protocols.bgp.utils.bgp \
    import clone_path_and_update_med_for_target_neighbor
LOG = logging.getLogger('bgpspeaker.core_managers.peer_manager')


class PeerManager(object):
    def __init__(
            self, core_service, neighbors_conf,
    ):
        self._core_service = core_service
        self._signal_bus = core_service.signal_bus
        self._table_manager = core_service.table_manager
        self._rt_manager = core_service.rt_manager
        self._peers = {}

        # Peer to RTFilter map
        # Key: Peer instance
        # Value: set of RTs that constitute RT filter for this peer
        self._peer_to_rtfilter_map = {}
        self._neighbors_conf = neighbors_conf

    @property
    def iterpeers(self):
        return iter(self._peers.values())

    def set_peer_to_rtfilter_map(self, new_map):
        self._peer_to_rtfilter_map = new_map

    def add_peer(self, neigh_conf, common_conf):
        peer = Peer(common_conf, neigh_conf, self._core_service,
                    self._signal_bus, self)
        self._peers[neigh_conf.ip_address] = peer
        self._core_service.on_peer_added(peer)

    def remove_peer(self, neigh_conf):
        neigh_ip_address = neigh_conf.ip_address
        peer = self._peers.get(neigh_ip_address)
        peer.stop()
        del self._peers[neigh_ip_address]
        self._core_service.on_peer_removed(peer)

    def get_by_addr(self, addr):
        return self._peers.get(str(netaddr.IPAddress(addr)))

    def on_peer_down(self, peer):
        """Peer down handler.

        Cleans up the paths in global tables that was received from this peer.
        """
        LOG.debug('Cleaning obsolete paths whose source/version: %s/%s',
                  peer.ip_address, peer.version_num)
        # Launch clean-up for each global tables.
        self._table_manager.clean_stale_routes(peer)

    def _get_non_rtc_peers(self):
        non_rtc_peer_list = set()
        for peer in self._peers.values():
            if (peer.in_established() and
                    not peer.is_mpbgp_cap_valid(RF_RTC_UC)):
                non_rtc_peer_list.add(peer)
        return non_rtc_peer_list

    def curr_peer_rtfilter(self, peer):
        return self._peer_to_rtfilter_map.get(peer)

    def get_peers_in_established(self):
        """Returns list of peers in established state."""
        est_peers = []
        for peer in self._peers.values():
            if peer.in_established:
                est_peers.append(peer)
        return est_peers

    def resend_sent(self, route_family, peer):
        """For given `peer` re-send sent paths.

        Parameters:
            - `route-family`: (RouteFamily) of the sent paths to re-send
            - `peer`: (Peer) peer for which we need to re-send sent paths
        """
        if peer not in self._peers.values():
            raise ValueError('Could not find given peer (%s)' % peer)

        if route_family not in SUPPORTED_GLOBAL_RF:
            raise ValueError(
                'Given route family (%s) is not supported.' % route_family
            )

        # Iterate over the global table for given afi, safi and enqueue
        # out-going routes.
        table = self._table_manager.get_global_table_by_route_family(
            route_family
        )

        for destination in table.values():
            # Check if this destination's sent - routes include this peer.
            # i.e. check if this destinations was advertised and enqueue
            # the path only if it was. If the current best-path has not been
            # advertised before, it might already have a OutgoingRoute queued
            # to be sent to the peer.
            sent_routes = destination.sent_routes
            if sent_routes is None or len(sent_routes) == 0:
                continue
            for sent_route in sent_routes:
                if sent_route.sent_peer == peer:
                    # update med - if previously med was set per neighbor or
                    # wasn't set at all now it could have changed and we may
                    # need to set new value there
                    p = sent_route.path
                    if p.med_set_by_target_neighbor or p.get_pattr(
                            BGP_ATTR_TYPE_MULTI_EXIT_DISC) is None:
                        sent_route.path = \
                            clone_path_and_update_med_for_target_neighbor(
                                sent_route.path, peer.med
                            )

                    ogr = OutgoingRoute(sent_route.path,
                                        for_route_refresh=True)
                    peer.enque_outgoing_msg(ogr)

    def req_rr_to_non_rtc_peers(self, route_family):
        """Makes refresh request to all peers for given address family.

        Skips making request to peer that have valid RTC capability.
        """
        assert route_family != RF_RTC_UC
        for peer in self._peers.values():
            # First check if peer is in established state
            if (peer.in_established and
                # Check if peer has valid capability for given address
                # family
                    peer.is_mbgp_cap_valid(route_family) and
                # Check if peer has valid capability for RTC
                    not peer.is_mbgp_cap_valid(RF_RTC_UC)):
                peer.request_route_refresh(route_family)

    def make_route_refresh_request(self, peer_ip, *route_families):
        """Request route-refresh for peer with `peer_ip` for given
        `route_families`.

        Will make route-refresh request for a given `route_family` only if such
        capability is supported and if peer is in ESTABLISHED state. Else, such
        requests are ignored. Raises appropriate error in other cases. If
        `peer_ip` is equal to 'all' makes refresh request to all valid peers.
        """
        LOG.debug('Route refresh requested for peer %s and route families %s',
                  peer_ip, route_families)
        if not SUPPORTED_GLOBAL_RF.intersection(route_families):
            raise ValueError('Given route family(s) % is not supported.' %
                             route_families)

        peer_list = []
        # If route-refresh is requested for all peers.
        if peer_ip == 'all':
            peer_list.extend(self.get_peers_in_established())
        else:
            given_peer = self._peers.get(peer_ip)
            if not given_peer:
                raise ValueError('Invalid/unrecognized peer %s' % peer_ip)
            if not given_peer.in_established:
                raise ValueError('Peer currently do not have established'
                                 ' session.')
            peer_list.append(given_peer)

        # Make route refresh request to valid peers.
        for peer in peer_list:
            peer.request_route_refresh(*route_families)

        return True

    def comm_all_rt_nlris(self, peer):
        """Shares/communicates current best rt_nlri paths with this peers.

        Can be used to send initial updates after we have established session
        with `peer` with which RTC capability is valid. Takes into account
        peers RTC_AS setting and filters all RT NLRIs whose origin AS do not
        match this setting.
        """
        # First check if for this peer mpbgp-rtc is valid.
        if not peer.is_mbgp_cap_valid(RF_RTC_UC):
            return

        neigh_conf = self._neighbors_conf.get_neighbor_conf(peer.ip_address)
        peer_rtc_as = neigh_conf.rtc_as
        # Iterate over all RT_NLRI destination communicate qualifying RT_NLRIs
        rtc_table = self._table_manager.get_rtc_table()
        for dest in rtc_table.values():
            best_path = dest.best_path
            # Ignore a destination that currently does not have best path
            if not best_path:
                continue

            # If this is a local path
            if best_path.source is None:
                # Check RT NLRI's origin AS matches peer RTC_AS setting
                origin_as = best_path.nlri.origin_as
                if origin_as == peer_rtc_as:
                    peer.communicate_path(best_path)
            else:
                # Communicate all remote RT NLRIs
                peer.communicate_path(best_path)

        # Also communicate EOR as per RFC
        peer.enque_end_of_rib(RF_RTC_UC)

    def comm_all_best_paths(self, peer):
        """Shares/communicates current best paths with this peers.

        Can be used to send initial updates after we have established session
        with `peer`.
        """
        LOG.debug('Communicating current best path for all afi/safi except'
                  ' 1/132')
        # We will enqueue best path from all global destination.
        for route_family, table in self._table_manager.iter:
            if route_family == RF_RTC_UC:
                continue
            if peer.is_mbgp_cap_valid(route_family):
                for dest in table.values():
                    if dest.best_path:
                        peer.communicate_path(dest.best_path)

    def comm_new_best_to_bgp_peers(self, new_best_path):
        """Communicates/enqueues given best path to be sent to all qualifying
        bgp peers.

        If this path came from iBGP peers, it is not sent to other iBGP peers.
        If this path has community-attribute, and if settings for recognize-
        well-know attributes is set, we do as per [RFC1997], and queue outgoing
        route only to qualifying BGP peers.
        """
        # Filter based on standard community
        # If new best path has community attribute, it should be taken into
        # account when sending UPDATE to peers.
        comm_attr = new_best_path.get_pattr(BGP_ATTR_TYPE_COMMUNITIES)
        if comm_attr:
            comm_attr_na = comm_attr.has_comm_attr(
                BGPPathAttributeCommunities.NO_ADVERTISE
            )
            # If we have NO_ADVERTISE attribute is present, we do not send
            # UPDATE to any peers
            if comm_attr_na:
                LOG.debug('New best path has community attr. NO_ADVERTISE = %s'
                          '. Hence not advertising to any peer', comm_attr_na)
                return

        qualified_peers = self._collect_peers_of_interest(
            new_best_path
        )

        # Distribute new best-path to qualified peers.
        for peer in qualified_peers:
            peer.communicate_path(new_best_path)

    def _collect_peers_of_interest(self, new_best_path):
        """Collect all peers that qualify for sharing a path with given RTs.
        """
        path_rts = new_best_path.get_rts()
        qualified_peers = set(self._peers.values())

        # Filter out peers based on RTC_AS setting if path is for RT_NLRI
        qualified_peers = self._rt_manager.filter_by_origin_as(
            new_best_path, qualified_peers
        )

        # We continue to filter out qualified peer based on path RTs
        # If new best path has RTs, we need to share this UPDATE with
        # qualifying peers
        if path_rts:
            # We add Default_RTC_NLRI to path RTs so that we can send it to
            # peers that have expressed interest in all paths
            path_rts.append(RouteTargetMembershipNLRI.DEFAULT_RT)
            # All peers that do not have RTC capability qualify
            qualified_peers = set(self._get_non_rtc_peers())
            # Peers that have RTC capability and have common RT with the path
            # also qualify
            peer_to_rtfilter_map = self._peer_to_rtfilter_map
            for peer, rt_filter in peer_to_rtfilter_map.items():
                # Ignore Network Controller (its not a BGP peer)
                if peer is None:
                    continue

                if rt_filter is None:
                    qualified_peers.add(peer)
                elif rt_filter.intersection(path_rts):
                    qualified_peers.add(peer)

        return qualified_peers

    def schedule_rr_to_non_rtc_peers(self):
        for route_family in SUPPORTED_GLOBAL_RF:
            # Since we are dealing with peers that do not support RTC,
            # ignore this address family
            if route_family == RF_RTC_UC:
                continue

            self.req_rr_to_non_rtc_peers(route_family)



from os_ken.services.protocols.bgp.operator.views.base import \
    create_dict_view_class
from os_ken.services.protocols.bgp.operator.views.base import OperatorDetailView
from os_ken.services.protocols.bgp.operator.views import fields


class ConfDetailView(OperatorDetailView):
    settings = fields.DataField('_settings')

    def encode(self):
        return self.get_field('settings')


ConfDictView = create_dict_view_class(ConfDetailView, 'ConfDictView')



from os_ken.services.protocols.bgp.operator.views.base import \
    create_dict_view_class
from os_ken.services.protocols.bgp.operator.views.base import \
    create_list_view_class
from os_ken.services.protocols.bgp.operator.views.base import OperatorDetailView
from os_ken.services.protocols.bgp.operator.views import fields

from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_AS_PATH
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_MULTI_EXIT_DISC
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_LOCAL_PREF
from os_ken.lib.packet.bgp import BGP_ATTR_TYPE_EXTENDED_COMMUNITIES


class CoreServiceDetailView(OperatorDetailView):
    rf_state = fields.RelatedViewField(
        'rf_state',
        'os_ken.services.protocols.bgp.operator.views.bgp.RfStateDetailView'
    )
    importmap_manager = fields.RelatedDictViewField(
        '_importmap_manager',
        'os_ken.services.protocols.bgp.operator'
        '.views.other.ImportmapManagerDetailView'
    )
    table_manager = fields.RelatedViewField(
        '_table_manager',
        'os_ken.services.protocols.bgp.operator.views.bgp.TableManagerDetailView'
    )
    peer_manager = fields.RelatedViewField(
        '_peer_manager',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerManagerDetailView'
    )
    router_id = fields.DataField('router_id')


class TableManagerDetailView(OperatorDetailView):
    tables = fields.RelatedDictViewField(
        '_tables',
        'os_ken.services.protocols.bgp.operator.views.bgp.TableDictView'
    )
    tables_for_rt = fields.RelatedDictViewField(
        '_tables_for_rt',
        'os_ken.services.protocols.bgp.operator.views.bgp.TableDictView'
    )
    global_tables = fields.RelatedDictViewField(
        '_global_tables',
        'os_ken.services.protocols.bgp.operator.views.bgp.TableDictView'
    )
    asbr_label_range = fields.DataField('_asbr_label_range')
    next_hop_label = fields.DataField('_next_hop_label')
    next_vpnv4_label = fields.DataField('_next_vpnv4_label')


class PeerManagerDetailView(OperatorDetailView):
    peers = fields.RelatedListViewField(
        '_peers',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDictView'
    )
    peers_summary = fields.RelatedListViewField(
        '_peers',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDictSummaryView'
    )


class RfStateDetailView(OperatorDetailView):
    pass


class PeerStateDetailView(OperatorDetailView):
    bgp_state = fields.DataField('_bgp_state')
    last_error = fields.DataField('_last_bgp_error')

    def encode(self):
        ret = super(PeerStateDetailView, self).encode()
        ret.update(self._obj.get_stats_summary_dict())
        return ret


class PeerDetailView(OperatorDetailView):
    remote_as = fields.DataField('remote_as')
    ip_address = fields.DataField('ip_address')
    enabled = fields.DataField('enabled')
    adj_rib_in = fields.RelatedViewField(
        'adj_rib_in',
        'os_ken.services.protocols.bgp.operator.views.bgp.ReceivedRouteDictView'
    )
    adj_rib_out = fields.RelatedViewField(
        'adj_rib_out',
        'os_ken.services.protocols.bgp.operator.views.bgp.SentRouteDictView'
    )
    neigh_conf = fields.RelatedViewField(
        '_neigh_conf',
        'os_ken.services.protocols.bgp.operator.views.conf.ConfDetailView'
    )
    common_conf = fields.RelatedViewField(
        '_common_conf',
        'os_ken.services.protocols.bgp.operator.views.conf.ConfDetailView'
    )
    state = fields.RelatedViewField(
        'state',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerStateDetailView'
    )

    def encode(self):
        ret = super(PeerDetailView, self).encode()
        ret.update({
            'stats': self.rel('state').encode(),
            'settings': self.rel('neigh_conf').encode()
        })
        return ret


class PeerDetailSummaryView(PeerDetailView):
    def encode(self):
        return {
            'conf': self.rel('neigh_conf').encode(),
            'info': self.rel('state').encode()
        }


class PeerRfDetailView(OperatorDetailView):
    rf = fields.DataField('rf')
    enabled = fields.DataField('enabled')
    peer = fields.RelatedViewField(
        'peer',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDetailView'
    )


class TableDetailView(OperatorDetailView):
    scope_id = fields.DataField('scope_id')
    route_family = fields.DataField('route_family')
    destinations = fields.RelatedDictViewField(
        '_destinations',
        'os_ken.services.protocols.bgp.operator.views.bgp.DestinationDictView'
    )


class PathDetailView(OperatorDetailView):
    source_version_num = fields.DataField('source_version_num')
    route_family = fields.RelatedViewField(
        'route_family',
        'os_ken.services.protocols.bgp.operator.views.bgp.RouteFamilyView'
    )
    nlri = fields.RelatedViewField(
        'nlri',
        'os_ken.services.protocols.bgp.operator.views.bgp.NlriDetailView'
    )
    is_withdraw = fields.DataField('is_withdraw')
    nexthop = fields.DataField('nexthop')
    pathattr_map = fields.DataField('pathattr_map')
    source = fields.RelatedViewField(
        'source',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDetailView'
    )

    def encode(self):
        ret = super(PathDetailView, self).encode()
        ret['nlri'] = self.rel('nlri').encode()
        ret['route_family'] = self.rel('route_family').encode()
        as_path = self.get_field('pathattr_map').get(BGP_ATTR_TYPE_AS_PATH)
        origin = self.get_field('pathattr_map').get(BGP_ATTR_TYPE_ORIGIN)
        metric = self.get_field('pathattr_map').get(
            BGP_ATTR_TYPE_MULTI_EXIT_DISC)
        local_pref = self.get_field('pathattr_map').get(
            BGP_ATTR_TYPE_LOCAL_PREF
        )

        ret['as_path'] = as_path.value if as_path else None
        ret['origin'] = origin.value if origin else None
        ret['metric'] = metric.value if metric else None
        ret['local_pref'] = local_pref.value if local_pref else None
        ext = ret['pathattr_map'].get(BGP_ATTR_TYPE_EXTENDED_COMMUNITIES)
        del ret['pathattr_map']
        if ext:
            ret['rt_list'] = ext.rt_list
            ret['soo_list'] = ext.soo_list
        return ret


class SentRouteDetailView(OperatorDetailView):
    timestamp = fields.DataField('timestamp')
    filtered = fields.DataField('filtered')
    path = fields.RelatedViewField(
        'path',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathDetailView',
    )
    peer = fields.RelatedViewField(
        'sent_peer',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDetailView'
    )

    def encode(self):
        ret = super(SentRouteDetailView, self).encode()
        ret.update({
            'path': self.rel('path').encode(),
        })
        return ret


class ReceivedRouteDetailView(OperatorDetailView):
    timestamp = fields.DataField('timestamp')
    filtered = fields.DataField('filtered')
    path = fields.RelatedViewField(
        'path',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathDetailView',
    )
    peer = fields.RelatedViewField(
        'received_peer',
        'os_ken.services.protocols.bgp.operator.views.bgp.PeerDetailView'
    )

    def encode(self):
        ret = super(ReceivedRouteDetailView, self).encode()
        ret.update({
            'path': self.rel('path').encode(),
        })
        return ret


class DestinationDetailView(OperatorDetailView):
    table = fields.RelatedViewField(
        '_table',
        'os_ken.services.protocols.bgp.operator.views.bgp.TableDetailView',
    )
    best_path = fields.RelatedViewField(
        'best_path',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathDetailView'
    )
    known_path_list = fields.RelatedListViewField(
        'known_path_list',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathListView'
    )
    new_path_list = fields.RelatedListViewField(
        '_new_path_list',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathListView'
    )
    withdraw_list = fields.RelatedListViewField(
        '_withdraw_list',
        'os_ken.services.protocols.bgp.operator.views.bgp.PathListView'
    )
    sent_routes = fields.RelatedListViewField(
        'sent_routes',
        'os_ken.services.protocols.bgp.operator.views.bgp.SentRouteListView'
    )
    nlri = fields.DataField('nlri')
    route_family = fields.DataField('route_family')


class IpNlriDetailView(OperatorDetailView):
    formatted_nlri = fields.DataField('formatted_nlri_str')
    prefix = fields.DataField('prefix')


class VpnNlriDetailView(IpNlriDetailView):
    labels = fields.DataField('label_list')
    rd = fields.DataField('route_dist')


class NlriDetailView(OperatorDetailView):
    def __new__(cls, obj, filter_func=None):
        from os_ken.lib.packet.bgp import LabelledVPNIPAddrPrefix
        from os_ken.lib.packet.bgp import LabelledVPNIP6AddrPrefix
        from os_ken.lib.packet.bgp import IPAddrPrefix, IP6AddrPrefix
        if isinstance(obj, (LabelledVPNIPAddrPrefix,
                            LabelledVPNIP6AddrPrefix)):
            return VpnNlriDetailView(obj)
        elif isinstance(obj, (IPAddrPrefix, IP6AddrPrefix)):
            return IpNlriDetailView(obj)
        else:
            return OperatorDetailView(obj, filter_func)

    def encode(self):
        return self._obj.formatted_nlri_str


class RouteFamilyView(OperatorDetailView):
    afi = fields.DataField('afi')
    safi = fields.DataField('safi')


##################################################################
# Declarations of list and dict views based on detail views above
##################################################################

PeerListView = create_list_view_class(PeerDetailView, 'PeerListView')
PeerDictView = create_dict_view_class(PeerDetailView, 'PeerDictView')

PeerListSummaryView = create_list_view_class(
    PeerDetailSummaryView,
    'PeerListSummaryView'
)

PeerDictSummaryView = create_dict_view_class(
    PeerDetailSummaryView,
    'PeerDictSummaryView'
)

TableDictView = create_dict_view_class(TableDetailView, 'TableDictView')


DestinationListView = create_list_view_class(
    DestinationDetailView, 'DestinationListView'
)

DestinationDictView = create_dict_view_class(
    DestinationDetailView, 'DestinationDictView'
)

PathListView = create_list_view_class(PathDetailView, 'PathListView')
PathDictView = create_dict_view_class(PathDetailView, 'PathDictView')

SentRouteListView = create_list_view_class(
    SentRouteDetailView,
    'SentRouteListView'
)

SentRouteDictView = create_dict_view_class(
    SentRouteDetailView,
    'SentRouteDictView'
)

ReceivedRouteDictView = create_dict_view_class(
    ReceivedRouteDetailView,
    'ReceivedRouteDictView'
)



# Copyright (C) 2014 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
 Import-map configuration.
"""
import logging

from os_ken.services.protocols.bgp.api.base import register
from os_ken.services.protocols.bgp.core_manager import CORE_MANAGER
from os_ken.services.protocols.bgp.core_managers.import_map_manager\
    import ImportMapAlreadyExistsError
from os_ken.services.protocols.bgp.rtconf.base import RuntimeConfigError

LOG = logging.getLogger('bgpspeaker.api.import_map')


@register(name='importmap.create')
def create_importmap(type, action, name, value, route_family=None):
    if action != 'drop':
        raise RuntimeConfigError(
            'Unknown action. For now we only support "drop" action.'
        )

    if type not in ('prefix_match', 'rt_match'):
        raise RuntimeConfigError(
            'Unknown type. We support only "prefix_match" and "rt_match".'
        )

    if type == 'prefix_match':
        return _create_prefix_match_importmap(name, value, route_family)
    elif type == 'rt_match':
        return _create_rt_match_importmap(name, value)


def _create_prefix_match_importmap(name, value, route_family):
    core_service = CORE_MANAGER.get_core_service()
    importmap_manager = core_service.importmap_manager
    try:
        if route_family == 'ipv4':
            importmap_manager.create_vpnv4_nlri_import_map(name, value)
        elif route_family == 'ipv6':
            importmap_manager.create_vpnv6_nlri_import_map(name, value)
        else:
            raise RuntimeConfigError(
                'Unknown address family %s. it should be ipv4 or ipv6'
                % route_family
            )
    except ImportMapAlreadyExistsError:
        raise RuntimeConfigError(
            'Map with this name already exists'
        )

    return True


def _create_rt_match_importmap(name, value):
    core_service = CORE_MANAGER.get_core_service()
    importmap_manager = core_service.importmap_manager
    try:
        importmap_manager.create_rt_import_map(name, value)
    except ImportMapAlreadyExistsError:
        raise RuntimeConfigError(
            'Map with this name already exists'
        )

    return True



from os_ken.services.protocols.bgp.signals import SignalBus


class BgpSignalBus(SignalBus):
    BGP_ERROR = ('error', 'bgp')
    BGP_DEST_CHANGED = ('core', 'dest', 'changed')
    BGP_VRF_REMOVED = ('core', 'vrf', 'removed')
    BGP_VRF_ADDED = ('core', 'vrf', 'added')
    BGP_NOTIFICATION_RECEIVED = ('bgp', 'notification_received')
    BGP_NOTIFICATION_SENT = ('bgp', 'notification_sent')
    BGP_VRF_STATS_CONFIG_CHANGED = (
        'core', 'vrf', 'config', 'stats', 'changed'
    )
    BGP_BEST_PATH_CHANGED = ('core', 'best', 'changed')
    BGP_ADJ_RIB_IN_CHANGED = ('core', 'adj', 'rib', 'in', 'changed')
    BGP_ADJ_RIB_OUT_CHANGED = ('core', 'adj', 'rib', 'out', 'changed')
    BGP_ADJ_UP = ('core', 'adj', 'up')
    BGP_ADJ_DOWN = ('core', 'adj', 'down')

    def bgp_error(self, peer, code, subcode, reason):
        return self.emit_signal(
            self.BGP_ERROR + (peer, ),
            {'code': code, 'subcode': subcode, 'reason': reason, 'peer': peer}
        )

    def bgp_notification_received(self, peer, notification):
        return self.emit_signal(
            self.BGP_NOTIFICATION_RECEIVED + (peer,),
            notification
        )

    def bgp_notification_sent(self, peer, notification):
        return self.emit_signal(
            self.BGP_NOTIFICATION_SENT + (peer,),
            notification
        )

    def dest_changed(self, dest):
        return self.emit_signal(
            self.BGP_DEST_CHANGED,
            dest
        )

    def vrf_removed(self, route_dist):
        return self.emit_signal(
            self.BGP_VRF_REMOVED,
            route_dist
        )

    def vrf_added(self, vrf_conf):
        return self.emit_signal(
            self.BGP_VRF_ADDED,
            vrf_conf
        )

    def stats_config_changed(self, vrf_conf):
        return self.emit_signal(
            self.BGP_VRF_STATS_CONFIG_CHANGED,
            vrf_conf
        )

    def best_path_changed(self, path, is_withdraw):
        return self.emit_signal(
            self.BGP_BEST_PATH_CHANGED,
            {'path': path, 'is_withdraw': is_withdraw})

    def adj_rib_in_changed(self, peer, received_route):
        return self.emit_signal(
            self.BGP_ADJ_RIB_IN_CHANGED,
            {'peer': peer, 'received_route': received_route})

    def adj_rib_out_changed(self, peer, sent_route):
        return self.emit_signal(
            self.BGP_ADJ_RIB_OUT_CHANGED,
            {'peer': peer, 'sent_route': sent_route})

    def adj_up(self, peer):
        return self.emit_signal(self.BGP_ADJ_UP, {'peer': peer})

    def adj_down(self, peer):
        return self.emit_signal(self.BGP_ADJ_DOWN, {'peer': peer})



# Copyright (C) 2012 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# vim: tabstop=4 shiftwidth=4 softtabstop=4

import six

import binascii
import unittest
from nose.tools import *
import struct
from os_ken import exception

from os_ken.ofproto import ofproto_common, ofproto_parser
from os_ken.ofproto import ofproto_v1_0, ofproto_v1_0_parser

import logging
LOG = logging.getLogger(__name__)

if six.PY3:
    buffer = bytes


class TestOfproto_Parser(unittest.TestCase):
    def setUp(self):
        LOG.debug('setUp')
        self.bufHello = binascii.unhexlify('0100000800000001')

        fr = '010600b0000000020000000000000abc' \
            + '00000100010000000000008700000fff' \
            + '0002aefa39d2b9177472656d61302d30' \
            + '00000000000000000000000000000000' \
            + '000000c0000000000000000000000000' \
            + 'fffe723f9a764cc87673775f30786162' \
            + '63000000000000000000000100000001' \
            + '00000082000000000000000000000000' \
            + '00012200d6c5a1947472656d61312d30' \
            + '00000000000000000000000000000000' \
            + '000000c0000000000000000000000000'
        self.bufFeaturesReply = binascii.unhexlify(fr)

        pi = '010a005200000000000001010040' \
            + '00020000000000000002000000000001' \
            + '080045000032000000004011f967c0a8' \
            + '0001c0a8000200010001001e00000000' \
            + '00000000000000000000000000000000' \
            + '00000000'
        self.bufPacketIn = binascii.unhexlify(pi)

    def tearDown(self):
        LOG.debug('tearDown')
        pass

    def testHello(self):
        (version,
         msg_type,
         msg_len,
         xid) = ofproto_parser.header(self.bufHello)
        eq_(version, 1)
        eq_(msg_type, 0)
        eq_(msg_len, 8)
        eq_(xid, 1)

    def testFeaturesReply(self):
        (version,
         msg_type,
         msg_len,
         xid) = ofproto_parser.header(self.bufFeaturesReply)

        msg = ofproto_parser.msg(self,
                                 version,
                                 msg_type,
                                 msg_len,
                                 xid,
                                 self.bufFeaturesReply)
        LOG.debug(msg)

        ok_(isinstance(msg, ofproto_v1_0_parser.OFPSwitchFeatures))
        LOG.debug(msg.ports[65534])
        ok_(isinstance(msg.ports[1], ofproto_v1_0_parser.OFPPhyPort))
        ok_(isinstance(msg.ports[2], ofproto_v1_0_parser.OFPPhyPort))
        ok_(isinstance(msg.ports[65534], ofproto_v1_0_parser.OFPPhyPort))

    def testPacketIn(self):
        (version,
         msg_type,
         msg_len,
         xid) = ofproto_parser.header(self.bufPacketIn)

        msg = ofproto_parser.msg(self,
                                 version,
                                 msg_type,
                                 msg_len,
                                 xid,
                                 self.bufPacketIn)
        LOG.debug(msg)
        ok_(isinstance(msg, ofproto_v1_0_parser.OFPPacketIn))

    @raises(AssertionError)
    def test_check_msg_len(self):
        (version,
         msg_type,
         msg_len,
         xid) = ofproto_parser.header(self.bufPacketIn)

        msg_len = len(self.bufPacketIn) + 1
        ofproto_parser.msg(self,
                           version,
                           msg_type,
                           msg_len,
                           xid,
                           self.bufPacketIn)

    @raises(exception.OFPUnknownVersion)
    def test_check_msg_parser(self):
        (version,
         msg_type,
         msg_len,
         xid) = ofproto_parser.header(self.bufPacketIn)

        version = 0xff
        ofproto_parser.msg(self,
                           version,
                           msg_type,
                           msg_len,
                           xid,
                           self.bufPacketIn)


class TestMsgBase(unittest.TestCase):
    """ Test case for ofproto_parser.MsgBase
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_init(self):
        pass

    def test_set_xid(self):
        xid = 3841413783
        c = ofproto_parser.MsgBase(object)
        c.set_xid(xid)
        eq_(xid, c.xid)

    @raises(AssertionError)
    def test_set_xid_check_xid(self):
        xid = 2160492514
        c = ofproto_parser.MsgBase(object)
        c.xid = xid
        c.set_xid(xid)

    def _test_parser(self, msg_type=ofproto_v1_0.OFPT_HELLO):
        version = ofproto_v1_0.OFP_VERSION
        msg_len = ofproto_v1_0.OFP_HEADER_SIZE
        xid = 2183948390
        data = b'\x00\x01\x02\x03'

        fmt = ofproto_v1_0.OFP_HEADER_PACK_STR
        buf = struct.pack(fmt, version, msg_type, msg_len, xid) \
            + data

        res = ofproto_v1_0_parser.OFPHello.parser(
            object, version, msg_type, msg_len, xid, bytearray(buf))

        eq_(version, res.version)
        eq_(msg_type, res.msg_type)
        eq_(msg_len, res.msg_len)
        eq_(xid, res.xid)
        eq_(buffer(buf), res.buf)

        # test __str__()
        list_ = ('version', 'msg_type', 'msg_len', 'xid')
        check = {}
        for s in str(res).rsplit(','):
            if '=' in s:
                (k, v,) = s.rsplit('=')
                if k in list_:
                    check[k] = v

        eq_(hex(ofproto_v1_0.OFP_VERSION), check['version'])
        eq_(hex(ofproto_v1_0.OFPT_HELLO), check['msg_type'])
        eq_(hex(msg_len), check['msg_len'])
        eq_(hex(xid), check['xid'])

        return True

    def test_parser(self):
        ok_(self._test_parser())

    @raises(AssertionError)
    def test_parser_check_msg_type(self):
        self._test_parser(ofproto_v1_0.OFPT_ERROR)

    def _test_serialize(self):

        class Datapath(object):
            ofproto = ofproto_v1_0
            ofproto_parser = ofproto_v1_0_parser

        c = ofproto_v1_0_parser.OFPHello(Datapath)

        c.serialize()
        eq_(ofproto_v1_0.OFP_VERSION, c.version)
        eq_(ofproto_v1_0.OFPT_HELLO, c.msg_type)
        eq_(0, c.xid)

        return True

    def test_serialize(self):
        ok_(self._test_serialize())


class TestMsgStrAttr(unittest.TestCase):
    """ Test case for ofproto_parser.msg_str_attr
    """

    def test_msg_str_attr(self):
        class Check(object):
            check = 'msg_str_attr_test'

        c = Check()
        buf = ''

        res = ofproto_parser.msg_str_attr(c, buf, ('check',))
        str_ = str(res)
        str_ = str_.rsplit()
        eq_('check', str_[0])
        eq_('msg_str_attr_test', str_[1])



from collections import namedtuple
import json
import logging
import pprint
import re
import six

(STATUS_OK, STATUS_ERROR) = range(2)

CommandsResponse = namedtuple('CommandsResponse', ['status', 'value'])

LOG = logging.getLogger('bgpspeaker.operator.command')


def default_help_formatter(quick_helps):
    """Apply default formatting for help messages

        :param quick_helps: list of tuples containing help info
     """
    ret = ''
    for line in quick_helps:
        cmd_path, param_hlp, cmd_hlp = line
        ret += ' '.join(cmd_path) + ' '
        if param_hlp:
            ret += param_hlp + ' '
        ret += '- ' + cmd_hlp + '\n'
    return ret


class Command(object):
    """Command class is used as a node in tree of commands.

    Each command can do some action or have some sub-commands, just like in IOS
    Command with it's sub-commands form tree.
    Each command can have one or more parameters. Parameters have to be
    distinguishable from sub-commands.
        One can inject dependency into command Cmd(api=my_object).
    This dependency will be injected to every sub-command. And can be used
    to interact with model/data etc.
        Example of path in command tree `show count all`.
    """

    help_msg = ''
    param_help_msg = ''
    command = ''
    cli_resp_line_template = '{0}: {1}\n'

    def __init__(self, api=None, parent=None,
                 help_formatter=default_help_formatter,
                 resp_formatter_name='cli'):
        """:param api: object which is saved as self.api
                 and re-injected to every sub-command. You can use it to
                 manipulate your model from inside Commands'
           :param parent: parent command instance.
           :param help_formatter: function used to format
                output of '?'command. Is re-injected to every
                sub-command as well.
           :param resp_formatter_name: used to select function to format
                output of _action. cli_resp_formatter and json_resp_formatter
                are defined by default, but you can define your own formatters.
                If you use custom formatter(not cli nor json) remember to
                implement it for every sub-command.
        """

        self.resp_formatter_name = resp_formatter_name

        if hasattr(self, resp_formatter_name + '_resp_formatter'):
            self.resp_formatter = \
                getattr(self, resp_formatter_name + '_resp_formatter')
        else:
            self.resp_formatter = self.cli_resp_formatter

        self.api = api
        self.parent_cmd = parent
        self.help_formatter = help_formatter
        if not hasattr(self, 'subcommands'):
            self.subcommands = {}

    def __call__(self, params):
        """You run command by calling it.

        :param params: As params you give list of subcommand names
            and params to final subcommand. Kind of like in
            cisco ios cli, ie. show int eth1 / 1, where show is command,
            int subcommand and eth1 / 1 is param for subcommand.
        :return: returns tuple of CommandsResponse and class of
            sub - command on which _action was called. (last sub - command)
            CommandsResponse.status is action status,
            and CommandsResponse.value is formatted response.
        """
        if len(params) == 0:
            return self._action_wrapper([])

        first_param = params[0]

        if first_param == '?':
            return self.question_mark()

        if first_param in self.subcommands:
            return self._instantiate_subcommand(first_param)(params[1:])

        return self._action_wrapper(params)

    @classmethod
    def cli_resp_formatter(cls, resp):
        """Override this method to provide custom formatting of cli response.
        """
        if not resp.value:
            return ''

        if resp.status == STATUS_OK:

            if type(resp.value) in (str, bool, int, float, six.text_type):
                return str(resp.value)

            ret = ''
            val = resp.value
            if not isinstance(val, list):
                val = [val]
            for line in val:
                for k, v in line.items():
                    if isinstance(v, dict):
                        ret += cls.cli_resp_line_template.format(
                            k, '\n' + pprint.pformat(v)
                        )
                    else:
                        ret += cls.cli_resp_line_template.format(k, v)
            return ret
        else:
            return "Error: {0}".format(resp.value)

    @classmethod
    def json_resp_formatter(cls, resp):
        """Override this method to provide custom formatting of json response.
        """
        return json.dumps(resp.value)

    @classmethod
    def dict_resp_formatter(cls, resp):
        return resp.value

    def _action_wrapper(self, params):
        filter_params = []
        if '|' in params:
            ind = params.index('|')
            new_params = params[:ind]
            filter_params = params[ind:]
            params = new_params

        action_resp = self.action(params)
        if len(filter_params) > 1:
            # we don't pass '|' around so filter_params[1:]
            action_resp = self.filter_resp(action_resp, filter_params[1:])
        action_resp = CommandsResponse(
            action_resp.status,
            self.resp_formatter(action_resp)
        )
        return action_resp, self.__class__

    def action(self, params):
        """Override this method to define what command should do.

        :param params: list of text parameters applied to this command.
        :return: returns CommandsResponse instance.
                 CommandsResponse.status can be STATUS_OK or STATUS_ERROR
                 CommandsResponse.value should be dict or str
        """
        return CommandsResponse(STATUS_ERROR, 'Not implemented')

    def filter_resp(self, action_resp, filter_params):
        """Filter response of action. Used to make printed results more
        specific

        :param action_resp: named tuple (CommandsResponse)
            containing response from action.
        :param filter_params: params used after '|' specific for given filter
        :return: filtered response.
        """
        if action_resp.status == STATUS_OK:
            try:
                return CommandsResponse(
                    STATUS_OK,
                    TextFilter.filter(action_resp.value, filter_params)
                )
            except FilterError as e:
                return CommandsResponse(STATUS_ERROR, str(e))
        else:
            return action_resp

    def question_mark(self):
        """Shows help for this command and it's sub-commands.
        """
        ret = []
        if self.param_help_msg or len(self.subcommands) == 0:
            ret.append(self._quick_help())

        if len(self.subcommands) > 0:
            for k, _ in sorted(self.subcommands.items()):
                command_path, param_help, cmd_help = \
                    self._instantiate_subcommand(k)._quick_help(nested=True)
                if command_path or param_help or cmd_help:
                    ret.append((command_path, param_help, cmd_help))

        return (
            CommandsResponse(STATUS_OK, self.help_formatter(ret)),
            self.__class__
        )

    def _quick_help(self, nested=False):
        """:param nested: True if help is requested directly for this command
                    and False when help is requested for a list of possible
                    completions.
        """
        if nested:
            return self.command_path(), None, self.help_msg
        else:
            return self.command_path(), self.param_help_msg, self.help_msg

    def command_path(self):
        if self.parent_cmd:
            return self.parent_cmd.command_path() + [self.command]
        else:
            return [self.command]

    def _instantiate_subcommand(self, key):
        return self.subcommands[key](
            api=self.api,
            parent=self,
            help_formatter=self.help_formatter,
            resp_formatter_name=self.resp_formatter_name
        )


class TextFilter(object):

    @classmethod
    def filter(cls, action_resp_value, filter_params):
        try:
            action, expected_value = filter_params
        except ValueError:
            raise FilterError('Wrong number of filter parameters')
        if action == 'regexp':

            if isinstance(action_resp_value, list):
                resp = list(action_resp_value)
                iterator = enumerate(action_resp_value)
            else:
                resp = dict(action_resp_value)
                iterator = iter(action_resp_value.items())

            remove = []

            for key, value in iterator:
                if not re.search(expected_value, str(value)):
                    remove.append(key)

            if isinstance(resp, list):
                resp = [resp[key] for key, value in enumerate(resp)
                        if key not in remove]
            else:
                resp = dict([(key, value)
                             for key, value in resp.items()
                             if key not in remove])

            return resp
        else:
            raise FilterError('Unknown filter')


class FilterError(Exception):
    pass



#!/usr/bin/env python

# Copyright (C) 2014 VA Linux Systems Japan K.K.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Fumihiko Kakuma, VA Linux Systems Japan K.K.
# @author: YAMAMOTO Takashi, VA Linux Systems Japan K.K.

# NOTE: This module is used by Neutron "ofagent" agent for
# IceHouse release.  Juno and later releases do not use this.
# TODO: Remove this module when IceHouse is EOL'ed.

from os_ken.lib import hub
hub.patch()

from os_ken import cfg

from neutron.common import config as logging_config

from os_ken.base.app_manager import AppManager


def main():
    cfg.CONF(project='os_ken')
    logging_config.setup_logging(cfg.CONF)
    AppManager.run_apps(['neutron.plugins.ofagent.agent.ofa_neutron_agent'])


if __name__ == "__main__":
    main()



# Copyright (C) 2013 Nippon Telegraph and Telephone Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# vim: tabstop=4 shiftwidth=4 softtabstop=4

import unittest
import logging
import six
import struct
import inspect
from nose.tools import ok_, eq_, nottest

from os_ken.ofproto import ether
from os_ken.lib.packet import packet
from os_ken.lib.packet import ethernet
from os_ken.lib.packet import lldp
from os_ken.lib import addrconv

LOG = logging.getLogger(__name__)


class TestLLDPMandatoryTLV(unittest.TestCase):
    def setUp(self):
        # sample data is based on:
        # http://wiki.wireshark.org/LinkLayerDiscoveryProtocol
        #
        # mandatory TLV only
        self.data = b'\x01\x80\xc2\x00\x00\x0e\x00\x04' \
                    + b'\x96\x1f\xa7\x26\x88\xcc\x02\x07' \
                    + b'\x04\x00\x04\x96\x1f\xa7\x26\x04' \
                    + b'\x04\x05\x31\x2f\x33\x06\x02\x00' \
                    + b'\x78\x00\x00'

    def tearDown(self):
        pass

    def test_get_tlv_type(self):
        buf = b'\x02\x07\x04\x00\x04\x96\x1f\xa7\x26'
        eq_(lldp.LLDPBasicTLV.get_type(buf), lldp.LLDP_TLV_CHASSIS_ID)

    def test_parse_without_ethernet(self):
        buf = self.data[ethernet.ethernet._MIN_LEN:]
        (lldp_pkt, cls, rest_buf) = lldp.lldp.parser(buf)
        eq_(len(rest_buf), 0)

        tlvs = lldp_pkt.tlvs
        eq_(tlvs[0].tlv_type, lldp.LLDP_TLV_CHASSIS_ID)
        eq_(tlvs[0].len, 7)
        eq_(tlvs[0].subtype, lldp.ChassisID.SUB_MAC_ADDRESS)
        eq_(tlvs[0].chassis_id, b'\x00\x04\x96\x1f\xa7\x26')
        eq_(tlvs[1].tlv_type, lldp.LLDP_TLV_PORT_ID)
        eq_(tlvs[1].len, 4)
        eq_(tlvs[1].subtype, lldp.PortID.SUB_INTERFACE_NAME)
        eq_(tlvs[1].port_id, b'1/3')
        eq_(tlvs[2].tlv_type, lldp.LLDP_TLV_TTL)
        eq_(tlvs[2].len, 2)
        eq_(tlvs[2].ttl, 120)
        eq_(tlvs[3].tlv_type, lldp.LLDP_TLV_END)

    def test_parse(self):
        buf = self.data
        pkt = packet.Packet(buf)
        i = iter(pkt)

        eq_(type(next(i)), ethernet.ethernet)
        eq_(type(next(i)), lldp.lldp)

    def test_tlv(self):
        tlv = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                             chassis_id=b'\x00\x04\x96\x1f\xa7\x26')
        eq_(tlv.tlv_type, lldp.LLDP_TLV_CHASSIS_ID)
        eq_(tlv.len, 7)
        (typelen, ) = struct.unpack('!H', b'\x02\x07')
        eq_(tlv.typelen, typelen)

    def test_serialize_without_ethernet(self):
        tlv_chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                        chassis_id=b'\x00\x04\x96\x1f\xa7\x26')
        tlv_port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                                  port_id=b'1/3')
        tlv_ttl = lldp.TTL(ttl=120)
        tlv_end = lldp.End()
        tlvs = (tlv_chassis_id, tlv_port_id, tlv_ttl, tlv_end)
        lldp_pkt = lldp.lldp(tlvs)

        eq_(lldp_pkt.serialize(None, None),
            self.data[ethernet.ethernet._MIN_LEN:])

    def test_serialize(self):
        pkt = packet.Packet()

        dst = lldp.LLDP_MAC_NEAREST_BRIDGE
        src = '00:04:96:1f:a7:26'
        ethertype = ether.ETH_TYPE_LLDP
        eth_pkt = ethernet.ethernet(dst, src, ethertype)
        pkt.add_protocol(eth_pkt)

        tlv_chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                        chassis_id=addrconv.mac.
                                        text_to_bin(src))
        tlv_port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                                  port_id=b'1/3')
        tlv_ttl = lldp.TTL(ttl=120)
        tlv_end = lldp.End()
        tlvs = (tlv_chassis_id, tlv_port_id, tlv_ttl, tlv_end)
        lldp_pkt = lldp.lldp(tlvs)
        pkt.add_protocol(lldp_pkt)

        eq_(len(pkt.protocols), 2)

        pkt.serialize()

        # Note: If ethernet frame is less than 60 bytes length,
        # ethernet.ethernet() appends padding to the payload.
        # So, we splits the serialized data to compare.
        data_len = len(self.data)
        pkt_data_lldp = pkt.data[:data_len]
        pkt_data_pad = pkt.data[data_len:]
        eq_(b'\x00' * (60 - data_len), pkt_data_pad)

        eq_(self.data, pkt_data_lldp)

    def test_to_string(self):
        chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                    chassis_id=b'\x00\x04\x96\x1f\xa7\x26')
        port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                              port_id=b'1/3')
        ttl = lldp.TTL(ttl=120)
        end = lldp.End()
        tlvs = (chassis_id, port_id, ttl, end)
        lldp_pkt = lldp.lldp(tlvs)

        chassis_id_values = {'subtype': lldp.ChassisID.SUB_MAC_ADDRESS,
                             'chassis_id': b'\x00\x04\x96\x1f\xa7\x26',
                             'len': chassis_id.len,
                             'typelen': chassis_id.typelen}
        _ch_id_str = ','.join(['%s=%s' % (k, repr(chassis_id_values[k]))
                               for k, v in inspect.getmembers(chassis_id)
                               if k in chassis_id_values])
        tlv_chassis_id_str = '%s(%s)' % (lldp.ChassisID.__name__, _ch_id_str)

        port_id_values = {'subtype': port_id.subtype,
                          'port_id': port_id.port_id,
                          'len': port_id.len,
                          'typelen': port_id.typelen}
        _port_id_str = ','.join(['%s=%s' % (k, repr(port_id_values[k]))
                                 for k, v in inspect.getmembers(port_id)
                                 if k in port_id_values])
        tlv_port_id_str = '%s(%s)' % (lldp.PortID.__name__, _port_id_str)

        ttl_values = {'ttl': ttl.ttl,
                      'len': ttl.len,
                      'typelen': ttl.typelen}
        _ttl_str = ','.join(['%s=%s' % (k, repr(ttl_values[k]))
                             for k, v in inspect.getmembers(ttl)
                             if k in ttl_values])
        tlv_ttl_str = '%s(%s)' % (lldp.TTL.__name__, _ttl_str)

        end_values = {'len': end.len,
                      'typelen': end.typelen}
        _end_str = ','.join(['%s=%s' % (k, repr(end_values[k]))
                             for k, v in inspect.getmembers(end)
                             if k in end_values])
        tlv_end_str = '%s(%s)' % (lldp.End.__name__, _end_str)

        _tlvs_str = '(%s, %s, %s, %s)'
        tlvs_str = _tlvs_str % (tlv_chassis_id_str,
                                tlv_port_id_str,
                                tlv_ttl_str,
                                tlv_end_str)

        _lldp_str = '%s(tlvs=%s)'
        lldp_str = _lldp_str % (lldp.lldp.__name__,
                                tlvs_str)

        eq_(str(lldp_pkt), lldp_str)
        eq_(repr(lldp_pkt), lldp_str)

    def test_json(self):
        chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                    chassis_id=b'\x00\x04\x96\x1f\xa7\x26')
        port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                              port_id=b'1/3')
        ttl = lldp.TTL(ttl=120)
        end = lldp.End()
        tlvs = (chassis_id, port_id, ttl, end)
        lldp1 = lldp.lldp(tlvs)
        jsondict = lldp1.to_jsondict()
        lldp2 = lldp.lldp.from_jsondict(jsondict['lldp'])
        eq_(str(lldp1), str(lldp2))


class TestLLDPOptionalTLV(unittest.TestCase):
    def setUp(self):
        # sample data is based on:
        # http://wiki.wireshark.org/LinkLayerDiscoveryProtocol
        #
        # include optional TLV
        self.data = b'\x01\x80\xc2\x00\x00\x0e\x00\x01' \
                    + b'\x30\xf9\xad\xa0\x88\xcc\x02\x07' \
                    + b'\x04\x00\x01\x30\xf9\xad\xa0\x04' \
                    + b'\x04\x05\x31\x2f\x31\x06\x02\x00' \
                    + b'\x78\x08\x17\x53\x75\x6d\x6d\x69' \
                    + b'\x74\x33\x30\x30\x2d\x34\x38\x2d' \
                    + b'\x50\x6f\x72\x74\x20\x31\x30\x30' \
                    + b'\x31\x00\x0a\x0d\x53\x75\x6d\x6d' \
                    + b'\x69\x74\x33\x30\x30\x2d\x34\x38' \
                    + b'\x00\x0c\x4c\x53\x75\x6d\x6d\x69' \
                    + b'\x74\x33\x30\x30\x2d\x34\x38\x20' \
                    + b'\x2d\x20\x56\x65\x72\x73\x69\x6f' \
                    + b'\x6e\x20\x37\x2e\x34\x65\x2e\x31' \
                    + b'\x20\x28\x42\x75\x69\x6c\x64\x20' \
                    + b'\x35\x29\x20\x62\x79\x20\x52\x65' \
                    + b'\x6c\x65\x61\x73\x65\x5f\x4d\x61' \
                    + b'\x73\x74\x65\x72\x20\x30\x35\x2f' \
                    + b'\x32\x37\x2f\x30\x35\x20\x30\x34' \
                    + b'\x3a\x35\x33\x3a\x31\x31\x00\x0e' \
                    + b'\x04\x00\x14\x00\x14\x10\x0e\x07' \
                    + b'\x06\x00\x01\x30\xf9\xad\xa0\x02' \
                    + b'\x00\x00\x03\xe9\x00\xfe\x07\x00' \
                    + b'\x12\x0f\x02\x07\x01\x00\xfe\x09' \
                    + b'\x00\x12\x0f\x01\x03\x6c\x00\x00' \
                    + b'\x10\xfe\x09\x00\x12\x0f\x03\x01' \
                    + b'\x00\x00\x00\x00\xfe\x06\x00\x12' \
                    + b'\x0f\x04\x05\xf2\xfe\x06\x00\x80' \
                    + b'\xc2\x01\x01\xe8\xfe\x07\x00\x80' \
                    + b'\xc2\x02\x01\x00\x00\xfe\x17\x00' \
                    + b'\x80\xc2\x03\x01\xe8\x10\x76\x32' \
                    + b'\x2d\x30\x34\x38\x38\x2d\x30\x33' \
                    + b'\x2d\x30\x35\x30\x35\x00\xfe\x05' \
                    + b'\x00\x80\xc2\x04\x00\x00\x00'

    def tearDown(self):
        pass

    def test_parse(self):
        buf = self.data
        pkt = packet.Packet(buf)
        i = iter(pkt)

        eq_(type(next(i)), ethernet.ethernet)
        lldp_pkt = next(i)
        eq_(type(lldp_pkt), lldp.lldp)

        tlvs = lldp_pkt.tlvs

        # Port Description
        eq_(tlvs[3].tlv_type, lldp.LLDP_TLV_PORT_DESCRIPTION)
        eq_(tlvs[3].port_description, b'Summit300-48-Port 1001\x00')

        # System Name
        eq_(tlvs[4].tlv_type, lldp.LLDP_TLV_SYSTEM_NAME)
        eq_(tlvs[4].system_name, b'Summit300-48\x00')

        # System Description

        eq_(tlvs[5].tlv_type, lldp.LLDP_TLV_SYSTEM_DESCRIPTION)
        eq_(tlvs[5].system_description,
            b'Summit300-48 - Version 7.4e.1 (Build 5) '
            + b'by Release_Master 05/27/05 04:53:11\x00')

        # SystemCapabilities
        eq_(tlvs[6].tlv_type, lldp.LLDP_TLV_SYSTEM_CAPABILITIES)
        eq_(tlvs[6].system_cap & lldp.SystemCapabilities.CAP_MAC_BRIDGE,
            lldp.SystemCapabilities.CAP_MAC_BRIDGE)
        eq_(tlvs[6].enabled_cap & lldp.SystemCapabilities.CAP_MAC_BRIDGE,
            lldp.SystemCapabilities.CAP_MAC_BRIDGE)
        eq_(tlvs[6].system_cap & lldp.SystemCapabilities.CAP_TELEPHONE, 0)
        eq_(tlvs[6].enabled_cap & lldp.SystemCapabilities.CAP_TELEPHONE, 0)

        # Management Address
        eq_(tlvs[7].tlv_type, lldp.LLDP_TLV_MANAGEMENT_ADDRESS)
        eq_(tlvs[7].addr_len, 7)
        eq_(tlvs[7].addr, b'\x00\x01\x30\xf9\xad\xa0')
        eq_(tlvs[7].intf_num, 1001)

        # Organizationally Specific
        eq_(tlvs[8].tlv_type, lldp.LLDP_TLV_ORGANIZATIONALLY_SPECIFIC)
        eq_(tlvs[8].oui, b'\x00\x12\x0f')  # IEEE 802.3
        eq_(tlvs[8].subtype, 0x02)  # Power Via MDI

        # End
        eq_(tlvs[16].tlv_type, lldp.LLDP_TLV_END)

    def test_parse_corrupted(self):
        buf = self.data
        pkt = packet.Packet(buf[:128])

    def test_serialize(self):
        pkt = packet.Packet()

        dst = lldp.LLDP_MAC_NEAREST_BRIDGE
        src = '00:01:30:f9:ad:a0'
        ethertype = ether.ETH_TYPE_LLDP
        eth_pkt = ethernet.ethernet(dst, src, ethertype)
        pkt.add_protocol(eth_pkt)

        tlv_chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                        chassis_id=addrconv.mac.
                                        text_to_bin(src))
        tlv_port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                                  port_id=b'1/1')
        tlv_ttl = lldp.TTL(ttl=120)
        tlv_port_description = lldp.PortDescription(
            port_description=b'Summit300-48-Port 1001\x00')
        tlv_system_name = lldp.SystemName(system_name=b'Summit300-48\x00')
        tlv_system_description = lldp.SystemDescription(
            system_description=b'Summit300-48 - Version 7.4e.1 (Build 5) '
                               + b'by Release_Master 05/27/05 04:53:11\x00')
        tlv_system_capabilities = lldp.SystemCapabilities(
            system_cap=0x14,
            enabled_cap=0x14)
        tlv_management_address = lldp.ManagementAddress(
            addr_subtype=0x06, addr=b'\x00\x01\x30\xf9\xad\xa0',
            intf_subtype=0x02, intf_num=1001,
            oid=b'')
        tlv_organizationally_specific = lldp.OrganizationallySpecific(
            oui=b'\x00\x12\x0f', subtype=0x02, info=b'\x07\x01\x00')
        tlv_end = lldp.End()
        tlvs = (tlv_chassis_id, tlv_port_id, tlv_ttl, tlv_port_description,
                tlv_system_name, tlv_system_description,
                tlv_system_capabilities, tlv_management_address,
                tlv_organizationally_specific, tlv_end)
        lldp_pkt = lldp.lldp(tlvs)
        pkt.add_protocol(lldp_pkt)

        eq_(len(pkt.protocols), 2)

        pkt.serialize()

        # self.data has many organizationally specific TLVs
        data = six.binary_type(pkt.data[:-2])
        eq_(data, self.data[:len(data)])

    def test_to_string(self):
        chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                    chassis_id=b'\x00\x01\x30\xf9\xad\xa0')
        port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                              port_id=b'1/1')
        ttl = lldp.TTL(ttl=120)
        port_desc = lldp.PortDescription(
            port_description=b'Summit300-48-Port 1001\x00')
        sys_name = lldp.SystemName(system_name=b'Summit300-48\x00')
        sys_desc = lldp.SystemDescription(
            system_description=b'Summit300-48 - Version 7.4e.1 (Build 5) '
                               + b'by Release_Master 05/27/05 04:53:11\x00')
        sys_cap = lldp.SystemCapabilities(
            system_cap=0x14,
            enabled_cap=0x14)
        man_addr = lldp.ManagementAddress(
            addr_subtype=0x06, addr=b'\x00\x01\x30\xf9\xad\xa0',
            intf_subtype=0x02, intf_num=1001,
            oid='')
        org_spec = lldp.OrganizationallySpecific(
            oui=b'\x00\x12\x0f', subtype=0x02, info=b'\x07\x01\x00')
        end = lldp.End()
        tlvs = (chassis_id, port_id, ttl, port_desc, sys_name,
                sys_desc, sys_cap, man_addr, org_spec, end)
        lldp_pkt = lldp.lldp(tlvs)

        # ChassisID string
        chassis_id_values = {'subtype': lldp.ChassisID.SUB_MAC_ADDRESS,
                             'chassis_id': b'\x00\x01\x30\xf9\xad\xa0',
                             'len': chassis_id.len,
                             'typelen': chassis_id.typelen}
        _ch_id_str = ','.join(['%s=%s' % (k, repr(chassis_id_values[k]))
                               for k, v in inspect.getmembers(chassis_id)
                               if k in chassis_id_values])
        tlv_chassis_id_str = '%s(%s)' % (lldp.ChassisID.__name__, _ch_id_str)

        # PortID string
        port_id_values = {'subtype': port_id.subtype,
                          'port_id': port_id.port_id,
                          'len': port_id.len,
                          'typelen': port_id.typelen}
        _port_id_str = ','.join(['%s=%s' % (k, repr(port_id_values[k]))
                                 for k, v in inspect.getmembers(port_id)
                                 if k in port_id_values])
        tlv_port_id_str = '%s(%s)' % (lldp.PortID.__name__, _port_id_str)

        # TTL string
        ttl_values = {'ttl': ttl.ttl,
                      'len': ttl.len,
                      'typelen': ttl.typelen}
        _ttl_str = ','.join(['%s=%s' % (k, repr(ttl_values[k]))
                             for k, v in inspect.getmembers(ttl)
                             if k in ttl_values])
        tlv_ttl_str = '%s(%s)' % (lldp.TTL.__name__, _ttl_str)

        # PortDescription string
        port_desc_values = {'tlv_info': port_desc.tlv_info,
                            'len': port_desc.len,
                            'typelen': port_desc.typelen}
        _port_desc_str = ','.join(['%s=%s' % (k, repr(port_desc_values[k]))
                                   for k, v in inspect.getmembers(port_desc)
                                   if k in port_desc_values])
        tlv_port_desc_str = '%s(%s)' % (lldp.PortDescription.__name__,
                                        _port_desc_str)

        # SystemName string
        sys_name_values = {'tlv_info': sys_name.tlv_info,
                           'len': sys_name.len,
                           'typelen': sys_name.typelen}
        _system_name_str = ','.join(['%s=%s' % (k, repr(sys_name_values[k]))
                                     for k, v in inspect.getmembers(sys_name)
                                     if k in sys_name_values])
        tlv_system_name_str = '%s(%s)' % (lldp.SystemName.__name__,
                                          _system_name_str)

        # SystemDescription string
        sys_desc_values = {'tlv_info': sys_desc.tlv_info,
                           'len': sys_desc.len,
                           'typelen': sys_desc.typelen}
        _sys_desc_str = ','.join(['%s=%s' % (k, repr(sys_desc_values[k]))
                                  for k, v in inspect.getmembers(sys_desc)
                                  if k in sys_desc_values])
        tlv_sys_desc_str = '%s(%s)' % (lldp.SystemDescription.__name__,
                                       _sys_desc_str)

        # SystemCapabilities string
        sys_cap_values = {'system_cap': 0x14,
                          'enabled_cap': 0x14,
                          'len': sys_cap.len,
                          'typelen': sys_cap.typelen}
        _sys_cap_str = ','.join(['%s=%s' % (k, repr(sys_cap_values[k]))
                                 for k, v in inspect.getmembers(sys_cap)
                                 if k in sys_cap_values])
        tlv_sys_cap_str = '%s(%s)' % (lldp.SystemCapabilities.__name__,
                                      _sys_cap_str)

        # ManagementAddress string
        man_addr_values = {'addr_subtype': 0x06,
                           'addr': b'\x00\x01\x30\xf9\xad\xa0',
                           'addr_len': man_addr.addr_len,
                           'intf_subtype': 0x02,
                           'intf_num': 1001,
                           'oid': '',
                           'oid_len': man_addr.oid_len,
                           'len': man_addr.len,
                           'typelen': man_addr.typelen}
        _man_addr_str = ','.join(['%s=%s' % (k, repr(man_addr_values[k]))
                                  for k, v in inspect.getmembers(man_addr)
                                  if k in man_addr_values])
        tlv_man_addr_str = '%s(%s)' % (lldp.ManagementAddress.__name__,
                                       _man_addr_str)

        # OrganizationallySpecific string
        org_spec_values = {'oui': b'\x00\x12\x0f',
                           'subtype': 0x02,
                           'info': b'\x07\x01\x00',
                           'len': org_spec.len,
                           'typelen': org_spec.typelen}
        _org_spec_str = ','.join(['%s=%s' % (k, repr(org_spec_values[k]))
                                  for k, v in inspect.getmembers(org_spec)
                                  if k in org_spec_values])
        tlv_org_spec_str = '%s(%s)' % (lldp.OrganizationallySpecific.__name__,
                                       _org_spec_str)

        # End string
        end_values = {'len': end.len,
                      'typelen': end.typelen}
        _end_str = ','.join(['%s=%s' % (k, repr(end_values[k]))
                             for k, v in inspect.getmembers(end)
                             if k in end_values])
        tlv_end_str = '%s(%s)' % (lldp.End.__name__, _end_str)

        # tlvs string
        _tlvs_str = '(%s, %s, %s, %s, %s, %s, %s, %s, %s, %s)'
        tlvs_str = _tlvs_str % (tlv_chassis_id_str,
                                tlv_port_id_str,
                                tlv_ttl_str,
                                tlv_port_desc_str,
                                tlv_system_name_str,
                                tlv_sys_desc_str,
                                tlv_sys_cap_str,
                                tlv_man_addr_str,
                                tlv_org_spec_str,
                                tlv_end_str)

        # lldp string
        _lldp_str = '%s(tlvs=%s)'
        lldp_str = _lldp_str % (lldp.lldp.__name__,
                                tlvs_str)

        eq_(str(lldp_pkt), lldp_str)
        eq_(repr(lldp_pkt), lldp_str)

    def test_json(self):
        chassis_id = lldp.ChassisID(subtype=lldp.ChassisID.SUB_MAC_ADDRESS,
                                    chassis_id=b'\x00\x01\x30\xf9\xad\xa0')
        port_id = lldp.PortID(subtype=lldp.PortID.SUB_INTERFACE_NAME,
                              port_id=b'1/1')
        ttl = lldp.TTL(ttl=120)
        port_desc = lldp.PortDescription(
            port_description=b'Summit300-48-Port 1001\x00')
        sys_name = lldp.SystemName(system_name=b'Summit300-48\x00')
        sys_desc = lldp.SystemDescription(
            system_description=b'Summit300-48 - Version 7.4e.1 (Build 5) '
                               + b'by Release_Master 05/27/05 04:53:11\x00')
        sys_cap = lldp.SystemCapabilities(
            system_cap=0x14,
            enabled_cap=0x14)
        man_addr = lldp.ManagementAddress(
            addr_subtype=0x06, addr=b'\x00\x01\x30\xf9\xad\xa0',
            intf_subtype=0x02, intf_num=1001,
            oid='')
        org_spec = lldp.OrganizationallySpecific(
            oui=b'\x00\x12\x0f', subtype=0x02, info=b'\x07\x01\x00')
        end = lldp.End()
        tlvs = (chassis_id, port_id, ttl, port_desc, sys_name,
                sys_desc, sys_cap, man_addr, org_spec, end)
        lldp1 = lldp.lldp(tlvs)
        jsondict = lldp1.to_jsondict()
        lldp2 = lldp.lldp.from_jsondict(jsondict['lldp'])
        eq_(str(lldp1), str(lldp2))
